from recipe_scrapers.albertheijn import AlbertHeijn
from tests import ScraperTest


class TestAlbertHeijnScraper(ScraperTest):

    scraper_class = AlbertHeijn

    def test_host(self):
        self.assertEqual("ah.nl", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Albert Heijn", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Kruidige groentecalzone", self.harvester_class.title())

    def test_category(self):
        self.assertEqual("hoofdgerecht", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://static.ah.nl/static/recepten/img_001329_890x594_JPG.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 kleine rode ui",
                "1 courgette",
                "1 rode paprika",
                "0.5 eetlepel Italiaanse kruiden",
                "2 eetlepels olijfolie",
                "2 theelepels knoflookpuree",
                "1 pakje pizza & tomato",
                "1 pakje walnootkaas",
                "0.5 eetlepel olie om in te vetten",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Oven voorverwarmen op 200 °C of gasovenstand 4. Ui, courgette en paprika schoonmaken en in kleine blokjes snijden. In kom groenten, kruiden, olijfolie, knoflookpuree en inhoud van potje tomatensaus door elkaar scheppen.Op aanrecht pizzadeeg uitspreiden, iets uitrollen en in twee gelijke stukken snijden. Plakken kaas naast elkaar erop leggen, ca. 2 cm van randen vrijhouden. Op helft van elk stuk courgettemengsel scheppen. Andere helft over vulling klappen en randen van deeg tussen duim en wijsvinger tot mooie schulprand dichtknijpen. Bakplaat invetten. Calzones erop leggen en in midden van oven in ca. 25 minuten goudbruin en gaarbakken.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(3.08, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("italiaans", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual("", self.harvester_class.description())



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class HEB(AbstractScraper):
    @classmethod
    def host(cls, domain="com"):
        return f"heb.{domain}"

    def title(self):
        return self.soup.find("h1", {"class": "title"}).get_text()

    def total_time(self):
        minutes_tag = self.soup.find("div", {"itemprop": "totalTime"})
        return get_minutes(minutes_tag.parent.get_text())

    def yields(self):
        yields_tag = self.soup.find("div", {"itemprop": "recipeYield"})
        return get_yields(yields_tag.parent.get_text())

    def ingredients(self):
        ingredients_container = self.soup.find(class_="ingredientswrapper")
        ingredients = ingredients_container.findAll("div", {"class": "recipestepstxt"})

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def _instructions_list(self):
        instructions_container = self.soup.find("div", {"class": "instructions"})
        instructions = instructions_container.findAll(
            "span", {"class": "instructiontxt"}
        )
        return [
            normalize_string(instruction.get_text()) for instruction in instructions
        ]

    def instructions(self):
        data = self._instructions_list()
        return "\n".join(data) if data else None

    def image(self):
        container = self.soup.find("div", {"class": "recipeimage"})
        if not container:
            return None

        image = container.find("img", {"src": True})
        return image["src"] if image else None



from recipe_scrapers.matprat import Matprat
from tests import ScraperTest


class TestMatprat(ScraperTest):

    scraper_class = Matprat

    def test_host(self):
        self.assertEqual("matprat.no", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.matprat.no/oppskrifter/gjester/butter-chicken---indisk-smorkylling/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Butter chicken - indisk smørkylling"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "MatPrat")

    def test_total_time(self):
        self.assertEqual(160, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://images.matprat.no/dxgehtetqy",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 stk. kylling (ca. 1300 g)",
                "4 båter hvitløk",
                "0,5 ss revet frisk ingefær",
                "2 ss sitronsaft",
                "1 dl gresk yoghurt",
                "0,5 ss chilipulver (helst indisk)",
                "1 ss garam masala",
                "1 ss rapsolje eller sennepsolje",
                "100 g cashewnøtter",
                "8 stk. tomat",
                "2 ss olje",
                "4 båter finhakket hvitløk",
                "0,5 ss revet frisk ingefær",
                "0,5 ss chilipulver (helst indisk)",
                "2 stk. hel kardemomme",
                "2 ts hel bukkehornkløver",
                "2 ss flytende honning",
                "1 stk. grønn chili",
                "2 ss smør",
                "1 dl fløte",
                "0,5 potte frisk koriander",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            """Fjern skinnet på kyllingen og tørk den godt. Kutt dype snitt i kjøttet på hele kyllingen. Finhakk hvitløk og bland med revet ingefær og sitronsaft, og gni kyllingen godt inn med blandingen. Dryss over litt salt. Sett kaldt i 20 minutter.\nBland sammen chilipulver, garam masala, olje og yoghurt. Tørk kyllingen med litt kjøkkenpapir og gni den godt inn med yoghurtblandingen. Plasser kyllingen på et fat og sett den kaldt i minst 4-6 timer, helst over natten.\nStek kyllingen midt i stekeovnen ved 200 °C, eller grill den til den er gyllenbrun og gjennomstekt, ca. 50 min. - 1 time.\nAvkjøl og plukk kjøttet av beina i store biter.\nLag smør- og tomatcurry: Bløtlegg cashewnøtter i litt lunkent vann i minst 30 minutter. Hell av vannet og mos nøttene i hurtigmikser eller med stavmikser. Sett til side.\nDel tomater i biter. Varm en sauteringspanne med olje og fres tomater, hvitløk, ingefær, chilipulver, Kardemomme og bukkehornkløver på middels varme til tomatene er helt myke.\nTilsett cashewpuré og bruk stavmikser eller hurtigmikser til å finmose sausen. Sil gjerne sausen gjennom en grov sikt for å få ut rester av skall, hvis du vil ha den ekstra fin.\nHa sausen tilbake i kjelen og la den småkoke i i noen minutter. Smak til med honning, finhakket grønn chili, salt og pepper. Visp inn fløte og romtemperert smør i den varme sausen.\nLegg kyllingbitene i sausen og la alt bli gjennomvarmt. Pynt med koriander.""",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
import functools
import logging

from recipe_scrapers.plugins._interface import PluginInterface
from recipe_scrapers.settings import settings

logging.basicConfig()
logger = logging.getLogger(__name__)


class TemplatePlugin(PluginInterface):
    """
    Sample starting point to write your custom plugin.

    Check the available plugins implementations for more details.
    """

    run_on_hosts = ("*",)
    run_on_methods = (
        "title",
        # ... others
    )

    @classmethod
    def run(cls, decorated):
        @functools.wraps(decorated)
        def decorated_method_wrapper(self, *args, **kwargs):
            # in here you'll have self.soup, self.schema and the other
            # instance attributes/methods you can work with.
            # check other plugins for examples
            logger.setLevel(settings.LOG_LEVEL)
            class_name = self.__class__.__name__
            method_name = decorated.__name__
            logger.debug(
                f"Decorating: {class_name}.{method_name}() with TemplatePlugin"
            )
            return decorated(self, *args, **kwargs)

        return decorated_method_wrapper



from recipe_scrapers.watchwhatueat import WatchWhatUEat
from tests import ScraperTest


class TestWatchWhatUEatScraper(ScraperTest):

    scraper_class = WatchWhatUEat

    def test_host(self):
        self.assertEqual("watchwhatueat.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.watchwhatueat.com/healthy-instant-pot-cauliflower-head/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Garlic And Herb Instant Pot Cauliflower With Delicious Gravy",
        )

    def test_author(self):
        self.assertEqual(
            self.harvester_class.author(), "Swati Kadam Gulati | Watch What U Eat"
        )

    def test_yields(self):
        self.assertEqual("5 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.watchwhatueat.com/wp-content/uploads/2018/10/Instant-Pot-Whole-Cauliflower-6.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 large cauliflower head",
                "3-4 garlic cloves finely chopped",
                "1 tbsp fresh thyme (or 1 tsp dried)",
                "1 tbsp fresh parsley finely chopped",
                "1/2 tbsp olive oil",
                "1 tbsp lemon juice",
                "1 tsp smoked paprika",
                "salt and pepper",
                "1 onion cut into large pieces",
                "1 tsp oil",
                "2-3 garlic cloves whole",
                "1 1/2 cup vegetable stock",
                "1/2 tsp thyme (or more if fresh)",
                "1/2 tbsp lemon juice",
                "1 tbsp whole wheat flour",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "In a small bowl combine all the ingredients listed under marinade.\nCut leaves from the bottom of the cauliflower head. Carefully trim the stem flush without cutting the florets. It will help the cauliflower to sit flat on the trivet.\nRub the prepared garlic-herb marinade on the cauliflower head evenly.\nFor the gravy heat oil in saute mode of the Instant Pot. Add onion and whole garlic cloves; cook until translucent.\nThen add vegetable stock, thyme and lemon juice. Place the trivet into the Instant Pot.\nPut the whole cauliflower head on the trivet. Cook for 3 min on manual high-pressure mode. Allow pressure to release naturally for 5 min and then quick release the remaining pressure.\nCarefully transfer the cauliflower to an oven-safe dish or tray. Optionally, broil it 3-4 min in the oven to get a nice crust.\nTurn on the saute mode in the Instant Pot. Add flour to the gravy base and blend everything using a hand blender until it gets to a smooth consistency. Add salt and pepper according to taste.\nCook the gravy for 3-4 min and serve warm with the cooked cauliflower.",
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper
from ._utils import normalize_string


class BricelEtBaklava(AbstractScraper):
    @classmethod
    def host(cls):
        return "briceletbaklava.ch"

    def title(self):
        return self.soup.find("h1", {"class": "Post-title"}).get_text().strip()

    def category(self):
        return "\n".join(
            [
                normalize_string(category.get_text())
                for category in self.soup.find_all("a", {"class": "Post-tag"})
            ]
        )

    def yields(self):
        post = self.soup.find("div", {"class": "Post-body"})
        section = post.find_all("div", {"class": "ob-section ob-section-html"})
        return normalize_string(section[1].find("p").get_text())

    def image(self):
        return self.soup.find("a", {"class": "ob-link-img"})["href"]

    def instructions(self):
        post = self.soup.find("div", {"class": "Post-body"})
        return "\n".join(
            [
                normalize_string(instruction.get_text())
                for instruction in post.find_all("ul")
            ]
        )

    def ingredients(self):
        post = self.soup.find("div", {"class": "Post-body"})
        sections = post.find_all("div", {"class": "ob-section ob-section-html"})
        return [
            normalize_string(ingredient.get_text())
            for ingredient in sections[1].find_all("b")
        ]

    def description(self):
        return normalize_string(
            self.soup.find("div", {"class": "ob-section ob-section-html"}).get_text()
        )

    def author(self):
        return "Michel/Bricelet & Baklava"



# mypy: disallow_untyped_defs=False
import functools
import logging

from recipe_scrapers.settings import settings

from ._interface import PluginInterface

logging.basicConfig()
logger = logging.getLogger(__name__)


class ExceptionHandlingPlugin(PluginInterface):
    """
    Plugin that is used only if settings.SUPPRESS_EXCEPTIONS is set to True.

    The outer-most plugin and decorator.

    If ANY of the methods listed raises ANY kind of exception, silence it
    and return the respective value from settings.ON_EXCEPTION_RETURN_VALUES

    If settings.SUPPRESS_EXCEPTIONS is set to False this plugin is ignored and
    does nothing. (In other words exceptions won't be handled and will bubble up
    to program's explosion. Left to the end-user to handle them on his own).
    """

    run_on_hosts = ("*",)
    run_on_methods = (
        "title",
        "total_time",
        "yields",
        "image",
        "ingredients",
        "instructions",
        "ratings",
        "reviews",
        "links",
        "language",
        "nutrients",
    )

    @classmethod
    def run(cls, decorated):
        @functools.wraps(decorated)
        def decorated_method_wrapper(self, *args, **kwargs):
            if settings.SUPPRESS_EXCEPTIONS:
                logger.setLevel(settings.LOG_LEVEL)
                class_name = self.__class__.__name__
                method_name = decorated.__name__
                logger.debug(
                    f"Decorating: {class_name}.{method_name}() with ExceptionHandlingPlugin"
                )

                try:
                    return decorated(self, *args, **kwargs)
                except Exception as e:
                    logger.info(
                        f"ExceptionHandlingPlugin silenced exception: {str(e)} in {class_name}.{method_name}()"
                    )

                    return settings.ON_EXCEPTION_RETURN_VALUES.get(
                        decorated.__name__, None
                    )
            return decorated(self, *args, **kwargs)

        return decorated_method_wrapper



# mypy: allow-untyped-defs

from recipe_scrapers.weightwatcherspublic import WeightWatchersPublic
from tests import ScraperTest


class TestweightwatchersPublicScraper(ScraperTest):

    scraper_class = WeightWatchersPublic

    # Test-Url:
    # https://www.weightwatchers.com/de/rezept/kartoffelgulasch/562a9b02873e1afb2a3c4c13

    def test_host(self):
        self.assertEqual("www.weightwatchers.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("WeightWatchers", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Kartoffelgulasch")

    def test_category(self):
        self.assertEqual("WeightWatchers", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_cook_time(self):
        self.assertEqual(0, self.harvester_class.cook_time())

    def test_prep_time(self):
        self.assertEqual(40, self.harvester_class.prep_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cmx.weightwatchers.com/assets-proxy/weight-watchers/image/upload/q_auto/h7wo0hbnwcleucj30sbw.jpg?auto=webp",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "800 g Kartoffeln; vorwiegend festkochend",
                "2 Stück Zwiebel/n; mittelgroß",
                "2 Stück Paprika; rot",
                "2 Stück Paprika; grün",
                "1 EL Petersilie; gehackt",
                "2 Stück Paprika; gelb",
                "250 g Tomaten, frisch",
                "4 Stück Wiener Würstchen",
                "2 TL Pflanzenöl, Rapsöl/Sonnenblumenöl",
                "2 EL Tomatenmark",
                "250 ml Gemüsebouillon/Gemüsebrühe, zubereitet; (1 TL Instantpulver)",
                "1 TL Oregano; gehackt",
                "1 TL Paprikapulver",
                "1 Prise(n) Salz/Jodsalz",
                "1 Prise(n) Pfeffer",
                "2 EL Schmand, 24 % Fett",
            ],
            self.harvester_class.ingredients(),
        )

    def test_ingredients_count(self):
        self.assertEqual(16, len(self.harvester_class.ingredients()))

    def test_instructions(self):
        self.assertEqual(
            "Kartoffeln und Zwiebeln schälen. Kartoffeln würfeln und Zwiebeln in Streifen schneiden. Paprika waschen, entkernen und in Stücke schneiden. Tomaten waschen und in Spalten schneiden. Würstchen in Scheiben schneiden.\nÖl in einem Topf erhitzen, Kartoffelwürfel und Zwiebelstreifen darin ca. 5 Minuten anbraten. Paprikastücke zufügen, Tomatenmark einrühren und kurz mitbraten. Mit Brühe ablöschen und zugedeckt ca. 15 Minuten garen.\nKartoffelgulasch mit Oregano, Paprikapulver, Salz und Pfeffer würzen. Würstchenscheiben und Tomatenspalten zufügen und weitere ca. 5 Minuten garen. Kartoffelgulasch mit Salz und Pfeffer abschmecken. Mit einem Klecks Schmand und Petersilie bestreut servieren.",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            "Das Rezept zaubert ein saftiges, würziges Gericht auf den Tisch und schmeckt garantiert.",
            self.harvester_class.description(),
        )

    def test_difficulty(self):
        self.assertEqual("Leicht", self.harvester_class.difficulty())

    def test_nutrients(self):
        expected_nutrients = {
            "points": "10 bis 13 PersonalPoints",
        }
        self.assertEqual(self.harvester_class.nutrients(), expected_nutrients)



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper
from ._utils import change_keys


def fix_json_ld_property_keys(k):
    if k == "ItemListElement":
        return "itemListElement"
    return k


class TineNo(AbstractScraper):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Fix incorrect property naming
        self.schema.data = change_keys(self.schema.data, fix_json_ld_property_keys)

    @classmethod
    def host(cls):
        return "tine.no"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        image = self.soup.find("img", {"id": "HeaderMediaContent"})
        if not image:
            # No image available
            return None
        # Prefer data src set for higher image resolution
        srcset_str = image.get("data-srcset", None)
        if not srcset_str:
            # Fallback to regular img source
            return image.get("src", None)
        # Convert source set string to cleaner list
        srcset = srcset_str.split(",")
        # Find image with the best resolution
        max_res = None
        max_res_src = None
        res_pattern = re.compile(r"(\d+)w")
        for src_str in srcset:
            src = src_str.strip().split(" ")
            res_match = res_pattern.findall(src[1])
            res = None
            if res_match:
                try:
                    res = int(res_match[0])
                except ValueError:
                    pass
            if max_res is None or res > max_res:
                max_res = res
                max_res_src = src[0]
        return max_res_src

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        """
        Standard Schema.org implementation, except removes HowToSection with generic 'Oppskrift' header.
        These are "pseudo" sections that are not rendered on the actual website.
        """
        return "\n".join(
            [i for i in self.schema.instructions().split("\n") if i != "Oppskrift"]
        )

    def ratings(self):
        return self.schema.ratings()

    def cuisine(self):
        return self.schema.cuisine()

    def description(self):
        return self.schema.description()



from recipe_scrapers.valdemarsro import Valdemarsro
from tests import ScraperTest


class TestValdemarsroScraper(ScraperTest):

    scraper_class = Valdemarsro

    def test_host(self):
        self.assertEqual("valdemarsro.dk", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            "Madpandekager",
            self.harvester_class.title(),
        )

    def test_category(self):
        self.assertEqual(
            "Bålmad,Familiefavoritter,Madpandekager og madvafler,Nem Hverdagsmad,Opskrifter,Opskrifter til børn - Hverdagsfavoritter børn,Pandekager,Tilbehør Aftensmad,Vegetar",
            self.harvester_class.category(),
        )

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_cook_time(self):
        self.assertEqual(20, self.harvester_class.cook_time())

    def test_yields(self):
        self.assertEqual("10 stk.", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.valdemarsro.dk/wp-content/2020/10/madpandekager.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 æg",
                "4 dl mælk",
                "1 nip salt",
                "125 g hvedemel",
                "50 g grahamsmel",
                "1 spsk smør, til stegning",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Pisk æg og mælk sammen med salt, grahamsmel og hvedemel.
Smelt lidt smør eller kom lidt olie på en pande og steg 10 madpandekager en ad gangen, på begge sider, til de er flotte lysebrune.""",
            self.harvester_class.instructions(),
        )

    def test_author(self):
        self.assertEqual("Ann-Christine Hellerup Brandt", self.harvester_class.author())

    def test_description(self):
        self.assertEqual(
            """Klassiske madpandekager er nemme og lækre til at fylde med alverdens gode sager og kan nydes til både frokost og aftensmad.
Man kan både fylde madpandekager ved bordet ala wraps eller man kan fylde dem med gode sager, rulle dem sammen og komme dem i et fad. Fordel lidt god friskrevet ost over og gratiner dem i ovnen. De er skønne at spise med en salat og dip til.
Jeg deler en masse ekstra tips til min opskrift på madpandekager under opskriften""",
            self.harvester_class.description(),
        )

    def test_language(self):
        self.assertEqual("da", self.harvester_class.language())

    def test_site_name(self):
        self.assertEqual("Valdemarsro", self.harvester_class.site_name())



from recipe_scrapers.altonbrown import AltonBrown
from tests import ScraperTest


class TestAltonBrownScraper(ScraperTest):

    scraper_class = AltonBrown

    def test_host(self):
        self.assertEqual("altonbrown.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Sarah Chanin", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("A Far, Far Better Cake", self.harvester_class.title())

    def test_category(self):
        self.assertEqual("Sweets", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(210, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://altonbrown.com/wp-content/uploads/2020/08/A-Far-Far-Better-Cake_RecipeImage.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 cup (2 sticks) unsalted butter, at room temperature, plus extra for the pan",
                "1 cup all-purpose flour, plus extra for the pan",
                "1 cup cake flour",
                "1 tablespoon baking powder",
                "1 1/2 cups plus 3 tablespoons sugar",
                "1/4 teaspoon fine sea salt",
                "8 large egg yolks, at room temperature",
                "1 1/4 cups whole milk, at room temperature",
                "1 teaspoon vanilla extract",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Place an oven rack in the top third of the oven and crank the box to 350°F. Then prep two 9-inch round aluminum cake pans by rubbing on a thin layer of butter on the bottoms and sides, and then flour. Finally, line the bottoms of both with a round of parchment paper.\nSiftYou can use a sifter for this or a fine sieve. the flours together with the baking powder and set aside.\nLoad the paddle attachment onto your stand mixer and beat the butter on low speed just until smooth, about 1 minute. Follow with the sugar and salt, boosting the speed to medium. Continue to beat until the mixture is light and fluffy, about 4 minutes, stopping to scrape down the sides of the bowl after each minute or so.\nWith the mixer still on medium, add the egg yolks one at a time, fully incorporating one before adding the next. When all eight are in, kill the mixer and scrape the bowl. Return the speed to low and slowly addI like to use a paper plate to dose in the flour...great multitasker. half the flour mixture.\nWhen the first half of the flour mix is worked in, add half of the milk and all of the vanilla. Stop and scrape the bowl yet again then return to low and slowly add the remaining flour. Then scrape one more time, making sure there’s no dry flour hiding in the bottom of the bowl. Congratulations, you’ve conquered the creaming method.\nDivide the mixture between the two pans. I use a scale to ensure even distribution and that means about 660 grams per pan. Smooth the tops with a rubber spatula and tap the pans against the counter to remove any air pockets.\nBake until the cakes reach an internal temperature of between 207 and 210°F, which typically takes between 30 and 35 minutes. The cakes should be golden brown and a toothpick inserted into the center of the cakes should come out clean.\nRemove from oven and cool cakes on a wire rack for 15 minutes then de-pan onto the rack and cool to room temp before frosting.""",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.3, self.harvester_class.ratings())

    def test_description(self):
        self.assertEqual(
            """Way back in 2003, I created a yellow cake that was dry, crumbly, and not nearly the cake the world deserves. After all these years I can finally say: we’re golden...cake that is. All we needed was more moisture and less starch, proof positive that small, balanced changes can radically improve or...disprove a cake.This recipe first appeared in Season 2 of Good Eats: Reloaded.""",
            self.harvester_class.description(),
        )



# mypy: disallow_untyped_defs=False
from bs4 import BeautifulSoup

from ._abstract import AbstractScraper
from ._utils import normalize_string


class KingArthur(AbstractScraper):
    @classmethod
    def host(cls):
        return "kingarthurbaking.com"

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        """
        King Arthur updated how they format their instructions to include html (instructions wrapped in <p>) in the
        `recipeInstructions`, parse the instructions assuming each step is wrapped in a <p> first, and fallback to just
        returning the schema instructions in case this is changed, again.
        """
        schema_instructions = self.schema.instructions()
        soup = BeautifulSoup(schema_instructions, "html.parser")
        instruction_elms = soup.findAll("p")
        if instruction_elms:
            return "\n".join(
                [normalize_string(elm.get_text()) for elm in instruction_elms]
            )
        return schema_instructions

    def ratings(self):
        return self.schema.ratings()



from recipe_scrapers.gonnawantseconds import GonnaWantSeconds
from tests import ScraperTest


class TestGonnaWantSeconds(ScraperTest):

    scraper_class = GonnaWantSeconds

    def test_host(self):
        self.assertEqual("gonnawantseconds.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.gonnawantseconds.com/white-chicken-enchiladas/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "White Chicken Enchiladas",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Kathleen")

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.gonnawantseconds.com/wp-content/uploads/2020/08/White-Chicken-Enchiladas-01.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "8-10 small flour tortillas",
                "3 cups cooked chicken (shredded or chopped)",
                "3 cups Monterey jack cheese (shredded-divided)",
                "3 tablespoons unsalted butter",
                "3 tablespoons flour",
                "2 cups chicken broth",
                "1 cup sour cream",
                "1 (4-ounce can) diced green chilies mild",
                "2-3 tablespoons green onions (sliced)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Spray a 9 X 13-inch baking dish with cooking spray and set aside. Preheat oven to 350 degrees.\nIn a small bowl, combine chicken and 1 cup of Monterey Jack cheese. Fill tortillas with this mixture and roll each one up then place seam side down in the prepared pan.\nMelt the butter in a skillet. Sprinkle flour over melted butter and whisk to combine. Cook for 1 minute to remove the flour taste. Remove the skillet from heat and whisk in broth. Place back on the heat and cook until the mixture has thickened and is bubbly. Cool sauce for 3-5 minutes. (Don't skip this step- if the sauce is too hot and you add the sour cream it will curdle it-yuck!) Add sour cream and chilies and stir until sauce is smooth and sour cream is completely dissolved.\nPour sauce over enchiladas and add remaining cheese over top. Bake in preheated oven for 20-25 minutes or until enchiladas are heated through and sauce is bubbly. Turn on the broiler and broil until the top is nicely golden. Top with chopped green onions and serve.",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            "Whip up the cheesiest, dreamiest dinner you've ever had with my sensational White Chicken Enchiladas! Guaranteed to please even the pickiest eaters!",
            self.harvester_class.description(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes


class RachlMansfield(AbstractScraper):
    @classmethod
    def host(cls):
        return "rachlmansfield.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def total_time(self):
        # Total time is not reported correctly by the schema data. Using the sum of the prep and cook times
        # as a workaround instead.
        prep_time = get_minutes(self.schema.data.get("prepTime")) or 0
        cook_time = get_minutes(self.schema.data.get("cookTime")) or 0
        return prep_time + cook_time

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_yields, normalize_string


class KwestiaSmaku(AbstractScraper):
    @classmethod
    def host(cls):
        return "kwestiasmaku.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def yields(self):
        return get_yields(
            self.soup.find("div", {"class": "field-name-field-ilosc-porcji"})
        )

    def image(self):
        return (
            self.soup.find("div", {"class": "view-zdjecia"})
            .find("li", {"class": "first"})
            .img["src"]
        )

    def ingredients(self):
        ingredients = self.soup.find(
            "div", {"class": "field-name-field-skladniki"}
        ).find_all("li")
        return [normalize_string(i.get_text()) for i in ingredients]

    def instructions(self):
        instructions = self.soup.find(
            "div", {"class": "field-name-field-przygotowanie"}
        ).find_all("li")
        return "\n".join([normalize_string(i.get_text()) for i in instructions])

    def ratings(self):
        return self.schema.ratings()



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper


class Coop(AbstractScraper):
    @classmethod
    def host(cls):
        return "coop.se"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def cook_time(self):
        return self.schema.cook_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        image = self.schema.image()
        if image is None or image == "":
            image = self.schema.data.get("image")
            if (
                image
                and isinstance(image, list)
                and len(image) >= 1
                and isinstance(image[0], str)
                and image[0].startswith("//")
            ):
                image = "https:" + image[0]
        return image

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()

    def cuisine(self):
        return self.schema.cuisine()

    def description(self):
        return self.schema.description()



from recipe_scrapers.bonappetit import BonAppetit
from tests import ScraperTest


class TestBonAppetitScraper(ScraperTest):

    scraper_class = BonAppetit

    def test_host(self):
        self.assertEqual("bonappetit.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.bonappetit.com/recipe/pork-chops-with-celery-and-almond-salad",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Pork Chops with Celery Salad")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Adam Rapoport")

    def test_total_time(self):
        self.assertEqual(None, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://assets.bonappetit.com/photos/59e4d7dc3279981dd6c79847/5:7/w_1936,h_2710,c_limit/pork-chops-with-celery-and-almond-salad.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "¼ cup dried unsweetened cranberries",
                "3 tablespoons unseasoned rice vinegar",
                "2 1½-inch-thick bone-in pork rib chops (about 1 pound each), patted dry",
                "Kosher salt",
                "4 tablespoons extra-virgin olive oil, divided",
                "3 sprigs thyme",
                "3 garlic cloves, smashed",
                "3 tablespoons unsalted butter, cut into pieces",
                "1 small shallot, finely chopped",
                "6 large or 8 medium celery stalks, thinly sliced on a diagonal",
                "½ cup parsley leaves with tender stems",
                "¼ cup chopped salted, dry-roasted almonds",
                "1 ounce Parmesan, shaved",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            'Combine cranberries and vinegar in a small bowl and set aside.\nSeason pork generously with salt, then rub with 1 Tbsp. oil total. Heat a dry medium skillet, preferably cast iron, over medium. Cook pork chops, moving once or twice to hotter areas of skillet, until first side is deeply browned, 6–9 minutes. Turn pork chops and cook until second sides are browned, about 5 minutes. Working one at a time, set chops on fatty side with tongs to melt and brown fat cap, about 1 minute each. At this point an instant-read thermometer inserted into the center of each chop should register 135°.\nAdd thyme, garlic, and butter to skillet and swirl to melt butter. Tilt skillet toward you so butter pools in the pan and spoon foaming butter over chops continuously until butter is browned, about 1 minute. Transfer pork chops, thyme, and garlic to a cutting board and let meat rest while you assemble the salad.\nCombine shallot and a couple of pinches of salt in a large bowl. Pour vinegar from reserved cranberries into bowl. Whisking constantly, gradually add remaining 3 Tbsp. oil. Add cranberries, celery, parsley, almonds, Parmesan, and several pinches of salt; toss to combine.\nCut along bones to remove meat from pork chops; slice meat ½" thick. Transfer meat and bones to a platter along with garlic and thyme, then drizzle any accumulated juices left on cutting board over top. Serve with salad.',
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper
from ._utils import get_yields


class RealSimple(AbstractScraper):
    @classmethod
    def host(cls):
        return "realsimple.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return get_yields(
            self.soup.find("div", string=re.compile(r"Yield:")).parent.get_text()
        )

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()



from recipe_scrapers.madewithlau import MadeWithLau
from tests import ScraperTest


class TestMadeWithLauScraper(ScraperTest):

    scraper_class = MadeWithLau

    def test_host(self):
        self.assertEqual("madewithlau.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Made With Lau", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Salt & Pepper Tofu (椒鹽豆腐)",
        )

    def test_description(self):
        self.assertEqual(
            "Fragrant deep-fried tofu that'll disappear from the table before you know it!",
            self.harvester_class.description(),
        )

    def cook_time(self):
        self.assertEqual(20, self.harvester_class.cook_time())

    def prep_time(self):
        self.assertEqual(40, self.harvester_class.prep_time())

    def total_time(self):
        self.assertEqual(60, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cdn.sanity.io/images/2r0kdewr/production/3172f1134f6ad9f09fafa7770328db9066b7215a-500x500.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "16 oz firm tofu",
                "3 stalks green onion",
                "2 oz mini bell pepper",
                "2 cloves garlic",
                "4 cups boiling water",
                "1 tsp salt",
                "0.5 egg",
                "6 tbsp cornstarch",
                "16 oz oil",
                "2 tbsp oil",
                "1 tbsp garlic salt",
                "0.25 tsp white pepper",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Step 1 - Cut ingredients\nFirst, cut to separate the green and the white parts of the green onions (3 stalks). You're only using the firmer, more pungent whites for this dish. Dice the green onion whites.\nHalve the bell peppers (2 oz) and discard the core and seeds. Cut into strips, and then dice them.\nSmash, peel, and finely mince the garlic (2 clove).\nStep 2 - Cut tofu\nDrain and rinse the firm tofu (16 oz). Cut the tofu into cubes.\n\nStep 3 - Boil tofu\nBoil hot water (4 cup) in a pot with the heat on high. Add salt (1 tsp) and stir to dissolve. When water is boiling, add the tofu cubes. Use chopsticks to shift the tofu around so they’re all fully submerged.\nCook for 3 minutes with the lid on, and then lower the heat to low and uncover the pot. Cook for another 2 minutes.\nCarefully pour the tofu into a colander to drain the water out. Let it start to cool.\nStep 4 - Dry & coat tofu\nLine a plate with a paper towel or a clean kitchen towel. Transfer the tofu cubes onto the lined plate in an even layer. Lay another towel on top and gently press to remove excess moisture.\nSpread a layer of cornstarch (4.8 tbsp) on a separate, large plate. Transfer the tofu into a bowl and add the beaten egg (0.50 ). Gently mix to evenly coat the tofu.\nTransfer the egg-covered tofu cubes to the cornstarch plate to begin coating them.\nSprinkle more cornstarch (1.2 tbsp) over the top of the tofu. Use chopsticks to flip and mix the tofu until all the pieces are evenly coated in cornstarch.\nStep 5 - Fry tofu\nAdd oil (16 oz) to a large pot and set the heat on high.\nAs the oil heats, mix together garlic salt (1 tbsp) and white pepper (0.25 tsp) in a separate bowl.\nPrepare a batch of dredged tofu in a spider strainer.\nWhen the oil reaches 400°F (or 200°C), carefully set the tofu in the oil. At the beginning, let the tofu cook undisturbed so that the crust can set.\nWhen the tofu has started to set and the oil has started to recover from the temperature drop, you can add the rest of the tofu. Set them down along the sides of the pot so they don't stick to the other pieces. Gently mix the pieces around so they fry evenly.\nWhen the tofu is golden brown, or after about 5 minutes of frying at 360°-380°F (180-190°C), turn off the heat and carefully remove the tofu. Let the excess oil drain from the tofu.\nStep 6 - Stir-fry everything\nHeat a wok on high. Add oil (2 tbsp) (take this from the deep-frying oil). Before the oil gets too hot (before it starts to shimmer or smoke), add the dried chili and stir immediately. Fry for just 8 seconds to extract the flavor, and then remove them.\nAdd the minced garlic and fry it for 15 seconds, stirring the entire time.\nAdd the diced bell peppers and green onion whites. Fry for 20-25 seconds, still stirring constantly.\nAdd the fried tofu and stir-fry for 10 seconds.\nWhile stirring constantly, sprinkle just enough of the garlic salt+white pepper seasoning mix over all the ingredients in the wok to evenly coat the tofu cubes.\nStep 7 - Plate & serve\nTurn off the heat and plate. Make sure to scoop up all the little bits of vegetables; they're delicious. Enjoy!",
            self.harvester_class.instructions(),
        )

    def test_cuisine(self):
        self.assertEqual("Chinese", self.harvester_class.cuisine())

    def test_category(self):
        self.assertEqual("side dish", self.harvester_class.category())



from recipe_scrapers.onehundredonecookbooks import OneHundredOneCookBooks
from tests import ScraperTest


class TestOneHundredOneCookBooksScraper(ScraperTest):

    scraper_class = OneHundredOneCookBooks

    def test_host(self):
        self.assertEqual("101cookbooks.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Heidi Swanson", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Coconut Broccoli Soup", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(20, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://images.101cookbooks.com/coconut_broccoli_soup.jpg?w=680&auto=format",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 14-ounce can of full fat coconut milk",
                "3 cloves garlic, smashed",
                "1 large yellow onion, chopped",
                "1 small serrano chile, stemmed and chopped",
                "2 teaspoons fine grain sea salt",
                "4 1/2 cups water",
                "2-3 large heads of broccoli (~1 1/2 lb.), cut into small florets",
                "2-3 large handfuls of spinach",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Scoop a big spoonful of thick coconut cream from the top of the coconut milk can. Add it to a large pan over medium-high heat. When hot, stir in the garlic, onions, chile, and salt. Sauté for a couple minutes, just long enough for everything to soften up. Add the remaining coconut milk, and the water, and bring to a simmer before adding the broccoli and spinach. Simmer just long enough for the broccoli to get tender throughout, 2 - 4 minutes. Immediately remove the soup from heat and puree with an immersion blender.  Add more water if you feel the need to thin the soup out. Taste and add more salt if needed.\nServe sprinkled with tofu cubes, toasted almonds, and lots of scallions.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(None, self.harvester_class.ratings())



from recipe_scrapers.bigoven import BigOven
from tests import ScraperTest


class TestBigOven(ScraperTest):

    scraper_class = BigOven

    def test_host(self):
        self.assertEqual("bigoven.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.bigoven.com/recipe/no-knead-herb-focaccia/2719857",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "No-Knead Herb Focaccia")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "BigOvenEditorial")

    def test_total_time(self):
        self.assertEqual(720, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("24 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://photos.bigoven.com/recipe/hero/no-knead-herb-focaccia-79674b.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 packet active dry yeast ; (or 2 ¼ teaspoons)",
                "2 cups warm water ; divided",
                "2 teaspoons granulated sugar",
                "5 ½ cups bread flour",
                "¼ cup extra-virgin olive oil ; (plus more to grease pans and dough)",
                "1 tablespoon sea salt (or kosher salt)",
                "flaky sea salt ; (like Maldon)",
                "¼ cup extra-virgin olive oil",
                "4 sprigs fresh rosemary leaves",
                "3 sprigs fresh thyme leaves",
                "2 sprigs fresh oregano ; (or various herbs of choice)",
                "¼ teaspoon crushed red pepper",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "\n".join(
                [
                    "Herb Topping:",
                    "In a small bowl, combine olive oil, herbs, and crushed red pepper. Set aside until ready.",
                    "For the dough:",
                    "1. In a small saucepan, warm 1 cup water to a lukewarm temperature of 105°F to 110°F on an instant-read thermometer. Pour water into the bowl of a stand mixer and gently whisk in yeast and sugar to dissolve. Allow the yeast to bloom briefly, about 5-10 minutes.",
                    "2. Add remaining cup water, flour, and sea salt. With the dough hook attachment on lowest speed, mix until a loose, craggy dough forms and then increase speed to medium-high and mix until a smooth dough begins to wrap around the hook, about 5 to 7 minutes. Turn off the mixer, cover bowl with a damp towel, and allow dough to rest for 10 to 15 minutes. Resume mixing on medium-high for 10 to 15 minutes until dough becomes very elastic and begins tugging away from the sides of the bowl. Dough will be quite sticky, but resist the urge to add additional flour -- dough.",
                    "3. For the first rise, drizzle ¼ cup olive oil into the mixing bowl and swirl the sides of the bowl to coat as well as the surface of the dough. Cover and allow to rise in a warm place until doubled in size, about 90 to 120 minutes.",
                    "4. Use this time to prepare your herb oil and prep your decorative toppings, if using.",
                    "5. For the second rise, drizzle another ¼ cup olive oil on a half sheet pan (or two quarter sheet pans). Then, with well-oiled hands, gingerly raise the dough from the mixing bowl and allow its weight to stretch the dough down upon itself. Repeat this stretching technique 4 to 6 times to help encourage extra rise. Transfer the dough to the sheet pan and stretch in all directions to coax it into rectangular submission. It will likely not comply straight away. Cover with oiled plastic wrap and, after a brief rest (about 10 to 15 minutes), stretch the dough a second time.",
                    "6. If you plan to delay baking, now is the time to cover the sheet pan with oiled plastic wrap and place in the refrigerator for up to 24 hours. Bring to room temperature about an hour before baking so that the dough has doubled in height.",
                    "7. While the dough is doubling, preheat the oven to 450°F. Arrange the oven racks to both their highest and lowest positions.",
                    "8. Once risen and ready to bake, uncover the dough and, with well-oiled hands, dimple the dough by plunging fingers spread wide downward into the bottom of the pan. Bubbles are good, but especially large ones can be gently deflated.",
                    "9. Drizzle the focaccia as desired. We recommend a bare minimum of ¼ extra-virgin olive oil and a generous flurry of flaky sea salt, but encourage you to gild the lily here with herbs and garlic aplenty.",
                    "10. Place focaccia on the lowest rack and bake until the edges begin to pull away from the sides and corners of the pan, about 15 to 20 minutes. Transfer to the top rack and continue baking until the top is golden brown and bubbles are very well browned, about 5 minutes. Cool in the pan briefly then transfer to a cooling rack until completely cooled.",
                    "Possible toppings to decorate:",
                    "red onions",
                    "shallots",
                    "chives",
                    "fresh sage",
                    "fresh thyme",
                    "fresh oregano",
                    "fresh dill",
                    "fresh parsley",
                    "fresh basil",
                    "mix of mini sweet peppers",
                    "bell peppers",
                    "banana peppers",
                    "jalapeno peppers",
                    "cherry tomatoes",
                    "kalamata olives, pitted",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual({"count": 1, "rating": 5.0}, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_yields


class AfghanKitchenRecipes(AbstractScraper):
    @classmethod
    def host(cls):
        return "afghankitchenrecipes.com"

    def author(self):
        given_name = self.soup.find("h5", {"class": "given-name"})
        return given_name and given_name.find("a", {"rel": "author"}).get_text()

    def title(self):
        return self.schema.title()

    def total_time(self):
        ready_in = self.soup.find("li", {"class": "ready-in"})
        if not ready_in:
            return
        ready_text = ready_in.find("span", {"class": "value"}).get_text()
        if not ready_text or not ready_text.endswith("h"):
            return
        hours, minutes = ready_text[:-2].split(":")
        return int(hours) * 60 + int(minutes)

    def yields(self):
        servings = self.soup.find("li", {"class": "servings"})
        if not servings:
            return
        return get_yields(servings)

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ingredient_elements = self.soup.findAll("li", {"class": "ingredient"})
        return [
            element.get_text() for element in ingredient_elements if element.get_text()
        ]

    def instructions(self):
        instruction_elements = self.soup.findAll("p", {"class": "instructions"})
        return "\n".join(
            [
                element.get_text().strip()
                for element in instruction_elements
                if element.get_text()
            ]
        )

    def ratings(self):
        rating = self.soup.find("meta", {"itemprop": "ratingValue"})
        return rating and rating.get("content") and round(float(rating["content"]), 2)



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper


class FredriksFikaAllas(AbstractScraper):
    @classmethod
    def host(cls):
        return "fredriksfika.allas.se"

    def title(self):
        return self.soup.find("h1").get_text()

    def category(self):
        return self.soup.find("div", {"class": "post_category"}).get_text()

    def image(self):
        return self.soup.find("meta", {"property": "og:image", "content": True}).get(
            "content"
        )

    def ingredients(self):
        ingredients = []
        content = self.soup.find("strong", string=re.compile("Ingredienser"))
        content_rows = str(content.parent).split("<br/>")

        for i in content_rows:
            if "Ingredienser" not in i:
                ingredients.append(i.replace("\r", "").replace("Gör så här:", ""))
            if "Gör så här" in i:
                break

        return ingredients

    def instructions(self):
        instructions = []
        content = self.soup.find("strong", string=re.compile("Gör så här"))

        content_rows = str(content.parent).split("<br/>")

        fill_data = False
        for i in content_rows:
            if fill_data:
                instructions.append(i.replace("\r", ""))
            if "Gör så här" in i:
                fill_data = True

        return "\n".join(instructions)



from recipe_scrapers.sallysbakingaddiction import SallysBakingAddiction
from tests import ScraperTest


class TestSallysBakingAddictionScraper(ScraperTest):

    scraper_class = SallysBakingAddiction

    def test_host(self):
        self.assertEqual("sallysbakingaddiction.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://sallysbakingaddiction.com/rosemary-garlic-pull-apart-bread/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Rosemary Garlic Pull Apart Bread"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Sally")

    def test_yields(self):
        self.assertEqual("1 serving", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cdn.sallysbakingaddiction.com/wp-content/uploads/2020/12/garlic-rosemary-pull-apart-bread-225x225.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 teaspoons Platinum Yeast by Red Star*",
                "1 Tablespoon granulated sugar",
                "3/4 cup (180ml) whole milk",
                "3 Tablespoons (45g) unsalted butter, softened to room temperature",
                "1 large egg",
                "2 and 1/3 cups (291g) all-purpose flour (spoon & leveled), plus more as needed*",
                "1 teaspoon salt",
                "1 teaspoon garlic powder",
                "1 Tablespoon finely chopped fresh rosemary (or 2 teaspoons dried)",
                "5 Tablespoons (75g) unsalted butter, extra soft (see note)",
                "1 Tablespoon finely chopped fresh rosemary (or 2 teaspoons dried)",
                "1 Tablespoon finely chopped fresh parsley (or 2 teaspoons dried)",
                "2 garlic cloves, minced or 1/2 teaspoon garlic powder",
                "1/4 teaspoon salt",
                "3/4 cup (95g) shredded parmesan, mozzarella, or white cheddar cheese (or your favorite shredded cheese)",
                "1 Tablespoon unsalted butter, melted",
                "flaky/coarse sea salt for sprinkling",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Make the dough\nPlace the yeast and sugar in the bowl of a stand mixer fitted with a dough hook or paddle attachment. Or, if you do not own a stand mixer, a regular large mixing bowl. Heat the milk on the stove or in the microwave until warm to touch, about 110°F (43°C). Pour warm milk on top of yeast/sugar. Whisk gently to combine, then loosely cover with a clean kitchen towel and allow to sit for 5-10 minutes. The mixture will be frothy after 5-10 minutes.\nIf you do not have a mixer, you can mix the dough together with a wooden spoon or rubber spatula in this step\nAdd the butter, egg, flour, salt, garlic powder, and rosemary. Beat on low speed for 3 minutes. Dough will be soft. Transfer it to a lightly floured work surface. Using lightly floured hands, knead it for 1 minute. If the dough is too sticky to handle, add 1-3 more Tablespoons of flour, but you want a very soft dough. Shape into a ball.\nPlace the dough in a greased bowl (I use nonstick spray to grease) and cover with plastic wrap or aluminum foil. Place in a slightly warm environment to rise until doubled in size, around 60-90 minutes. (If desired, use my warm oven trick for rising. See my answer to Where Should Dough Rise? in my Baking with Yeast Guide.)\nAs the dough rises, prepare the filling in the next step and grease a 9×5 inch loaf pan.\nMake the filling\nIn a medium bowl, mix the soft butter, rosemary, parsley, garlic, and salt together. If the butter is soft enough, you can just mix it all together with a spoon or fork. You can use an electric mixer if that’s easier too. Cover tightly and set aside until ready to use. (Don’t refrigerate unless making well in advance. It’s easiest to spread on the dough when at room temperature. If refrigerated, let it come to room temperature before spreading on dough pieces.)\nAssemble the bread:\nPunch down the dough to release the air. Place dough on a lightly floured work surface. Divide it into 12 equal pieces, each about 1/4 cup of dough and a little larger than a golf ball. Using lightly floured hands, flatten each into a circle that’s about 4 inches in diameter. The circle doesn’t have to be perfectly round. I do not use a rolling pan to flatten, but you certainly can if you want. Spread 1-2 teaspoons of filling mixture onto each. Sprinkle each with 1 Tablespoon of cheese. Fold circles in half and line in prepared baking pan, round side up. See photos above for a visual.\nCover with plastic wrap or aluminum foil and allow to rise once again in a slightly warm environment until puffy, about 45 minutes.\nAdjust the oven rack to the lower third position then preheat oven to 350°F (177°C).\nBake until golden brown, about 50 minutes. If you find the top of the loaf is browning too quickly, tent with aluminum foil. (Don’t be alarmed if there’s melted butter around the sides of the bread as it bakes, it will seep into the bread before it finishes.) Remove from the oven and place the pan on a wire rack. If desired, brush with melted butter for topping and sprinkle with sea salt.\nCool for 10 minutes in the pan, then remove from the pan and serve warm.\nCover and store leftovers at room temperature for up to 2 days or in the refrigerator for up to 1 week. Since the bread is extra crispy on the exterior, it will become a little hard after day 1. Reheat in a 300°F (149°C) oven for 10-15 minutes until interior is soft again or warm in the microwave.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.vegolosi import Vegolosi
from tests import ScraperTest


class TestVegolosiScraper(ScraperTest):

    scraper_class = Vegolosi

    def test_host(self):
        self.assertEqual("vegolosi.it", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.vegolosi.it/ricette-vegane/polpette-di-tofu-e-piselli-salsa-allo-zafferano/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Polpette di tofu e piselli con salsa allo zafferano",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Vegolosi.it")

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "250 g tofu al naturale",
                "150 g piselli lessati",
                "100 g cous cous",
                "100 ml acqua bollente",
                "20 g di mandorle",
                "20 g di olio extravergine di oliva",
                "basilico",
                "sale e pepe",
                "250 ml latte di soia senza zucchero ( oppure brodo vegetale o latte di avena o farro)",
                "15 g amido di mais o fecola di patate",
                "1 bustina zafferano in polvere",
                "Sale",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Si cucina! Dopo aver messo il cous cous in una ciotola capiente o in una pentola, versateci sopra l’acqua bollente e lasciate riposate il tempo necessario affinché quest’ultimo assorba tutto il liquido. Dopo qualche minuto con i rebbi di una forchetta sgranatelo per bene e lasciatelo da parte. Formiamo le polpettine Nel robot da cucina aggiungete il tofu spezzettato grossolanamente, il basilico, i piselli, metà del cous cous cotto, l’olio extravergine di oliva, il sale e il pepe. Tritate tutti gli ingredienti sino a ottenere un composto che potrete lavorare con le mani, quindi versatelo in una ciotola insieme al cous cous rimanente e mescolate. Correggete di sale, se ce ne fosse bisogno, e se il composto risultasse troppo asciutto, aggiungeteci ancora un filo di olio extravergine di oliva. In forno Con le mani umide formate con l’impasto dei bocconcini: la grandezza dovrà essere, più o meno, quella di una noce. Passateli poi in un trito fine di mandorle, che potrete realizzare sia al coltello che con un tritatutto. Procedete adagiando le polpette di tofu su una leccarda ricoperta da carta da forno. Infornate a 180 gradi per circa 15 minuti. Negli ultimi minuti utilizzate la funzione grill del forno per creare una giusta doratura delle vostre polpette vegane. Prepariamo la crema Nel frattempo, preparate la crema allo zafferano unendo tutti gli ingredienti, quindi il latte di soia, l’amido di mais, lo zafferano e il sale in un pentolino. Cuocete fino a raggiungere al consistenza desiderata. Servite le polpette vegan ben calde e con la loro salsa come accompagnamento.",
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

from recipe_scrapers.template import Template
from tests import ScraperTest


class TestTemplateScraper(ScraperTest):

    scraper_class = Template

    def test_host(self):
        self.assertEqual("example.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual(None, self.harvester_class.author())

    def test_title(self):
        self.assertEqual(None, self.harvester_class.title())

    def test_category(self):
        self.assertEqual(None, self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(None, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual(None, self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(None, self.harvester_class.image())

    def test_ingredients(self):
        self.assertEqual(None, self.harvester_class.ingredients())

    def test_instructions(self):
        self.assertEqual(None, self.harvester_class.instructions())

    def test_ratings(self):
        self.assertEqual(None, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual(None, self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(None, self.harvester_class.description())



# mypy: allow-untyped-defs

from recipe_scrapers.justonecookbook import JustOneCookbook
from tests import ScraperTest


class TestJustOneCookbookScraper(ScraperTest):

    scraper_class = JustOneCookbook

    def test_host(self):
        self.assertEqual("justonecookbook.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Namiko Chen", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Yaki Onigiri (Grilled Rice Ball)", self.harvester_class.title()
        )

    def test_category(self):
        self.assertEqual("Side Dish", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.justonecookbook.com/wp-content/uploads/2020/03/Yaki-Onigiri-Grilled-Rice-Ball-9532-I.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 rice cooker cups uncooked Japanese short-grain rice (2 rice-cooker-cups (180 ml x 2 = 360 ml) yields roughly 4 servings (3 ½ US cups) or 6 rice balls; see how to cook short-grain rice with a rice cooker, a pot over the stove, an instant pot, or a donabe)",
                "400 ml water",
                "1 Tbsp kosher salt (Diamond Crystal; use half for table salt) (you do not need to use all of it)",
                "1 Tbsp neutral-flavored oil (vegetable, rice bran, canola, etc.)",
                "soy sauce (I love using my homemade unagi sauce)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Gather all the ingredients. Cook the rice with a rice cooker, a pot over the stove, an instant pot, or a donabe. Let the cooked rice cool a little bit until you can hold rice without burning your hands. Do not let the rice completely cool down.\nTo Make Rice Balls\nFirst, wet both of your hands with water so the rice won’t stick.\nThen put some salt in your hands and rub to spread all around.\nScoop about a half cup of rice onto your palm.\nCover the rice with the other hand and gently form the rice into a triangle.\nMake sure the covering hand (my right hand) should be forming a triangle shape. When forming the onigiri shape, your hands should be just firm enough so the onigiri doesn't fall apart. You don't want to squeeze the rice too tight.\nI use three fingers (thumb, index finger, middle finger) to cover the area to make a nice triangle shape. Then rotate onigiri to make a perfect triangle.\nWhile you squeeze onigiri firmly with both hands, one of your hands has to press onigiri to keep a nice form.\nGently squeeze the center of the triangle on both sides so there is a slight indentation (for grilling onigiri). Now onigiri is ready! You can tell I’m not a good onigiri maker – no matter how many years I have been practicing.\nTo Grill Rice Ball\nLightly oil a cast-iron skillet and put it on medium heat.\nGrill onigiri until all sides are crispy and lightly browned. Rice will release itself when it forms a nice crust. Don’t flip it quickly. Just work on one side at a time and avoid turning over frequently, which may end up breaking into pieces.\nOnce all nicely toasted and lightly brown, lower heat to medium-low heat. Brush all sides with soy sauce (or unagi sauce). Rotate to make sure all sides become crispy. Be careful not to burn onigiri after you brush it with the sauce.\nTo Store\nRice gets hard when you refrigerate. You can individually wrap the Yaki Onigiri in plastic wrap and cover them with a thick kitchen towel and store in the refrigerator for up to 2 days. The towel will prevent the rice from getting too cold and keep the food stay cool but not cold. When you're ready to eat, bring it back to room temperature and reheat in a microwave or frying pan.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.66, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("Japanese", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "A favorite at Izakaya restaurants, Yaki Onigiri are Japanese grilled rice balls covered in savory soy sauce. With a crispy crust on the outside and soft sticky rice on the inside, these rice balls are simply irresistible and easy to make at home!",
            self.harvester_class.description(),
        )



from recipe_scrapers.vegrecipesofindia import VegRecipesOfIndia
from tests import ScraperTest


class TestVegRecipesOfIndiaScraper(ScraperTest):

    scraper_class = VegRecipesOfIndia

    def test_host(self):
        self.assertEqual("vegrecipesofindia.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Dassana Amit", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Sarson ka Saag (Authentic Punjabi Style)", self.harvester_class.title()
        )

    def test_total_time(self):
        self.assertEqual(240, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.vegrecipesofindia.com/wp-content/uploads/2012/12/sarson-ka-saag-recipe-1.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 bunch mustard greens ((sarson))",
                "½ bunch bathua leaves ((chenopodium/goosefoot/melde))",
                "½ bunch spinach leaves ((palak))",
                "1 cup chopped radish leaves (- use tender leaves, (mooli ke patte))",
                "3 to 4 inches white radish root",
                "1 cup fenugreek leaves (- chopped)",
                "1 cup chopped onions (or 2 medium sized onions)",
                "1.5 cups chopped tomatoes (or 3 medium-sized tomatoes)",
                "2 inches ginger (- chopped)",
                "2 green chilies (- chopped)",
                "7 to 8 garlic (- medium sized, chopped)",
                "½ teaspoon red chili powder",
                "2 to 3 pinches asafoetida (or ⅛ teaspoon asafoetida powder (hing))",
                "2 to 3 cups water (or add as required)",
                "2 tablespoon maize flour (or fine cornmeal)",
                "salt (as required)",
                "⅓ cup finely chopped onions (or 1 small to medium-sized onion)",
                "1 to 2 tablespoons oil (or ghee)",
                "2 cups cooked saag (or as required)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Making sarson ka saag\nFirstly clean and chop all the greens. Then wash or rinse the greens very well in running water.\nIn a 5 litre stovetop pressure cooker or pan add all the ingredients listed under "for sarson ka saag" except for maize flour.\nCover the pressure cook and cook for 6 to 7 minutes or more on medium-high heat.\nIf cooking in a pan, then cover and let the greens cook till tender and softened. Do check occasionally.\nPour the greens along with the stock and maize flour in a blender. Blend till smooth.\nIn another deep pan or in the same cooker, pour the pureed greens.\nSimmer for a good 25 to 30 minutes on a low heat stirring at intervals.\nTempering for sarson da saag\nIn another small pan, heat oil. Use any neutral oil. You can also make the tempering with ghee if you prefer.\nAdd the chopped onions and saute them till light brown on medium-low heat.\nAdd the prepared saag. Stir and simmer for a couple of minutes.\nStir ocaasionally when the saag is simmering.\nServing Suggestions\nServe sarson ka saag hot with a side of some chopped onions, whole green chilies or mango pickle. Top the saag with a dollop of white butter and serve with Makki di Roti. Also serve a few jaggery cubes by the side.\nFor a vegan saag, omit adding white butter or use vegan butter.\nStorage\nStore the pureed and simmered saag in an airtight container for 4 to 5 days in the refrigerarot. You can also freeze it for a month.\nWhen ready to serve, remove the quantity you need from the refrigerator and temper it with the required amout of oil and onions.""",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.92, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper


class HelloFresh(AbstractScraper):
    @classmethod
    def host(cls, domain="com"):
        return f"hellofresh.{domain}"

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def ingredients(self):
        if not self._serving_one_on_page():
            return self.schema.ingredients()
        else:
            return [f"2 * {ingredient}" for ingredient in self.schema.ingredients()]

    def instructions(self):
        return self.schema.instructions()

    def image(self):
        return self.schema.image()

    def nutrients(self):
        return self.schema.nutrients()

    def cuisine(self):
        return self.schema.cuisine()

    def category(self):
        return self.schema.category()

    def _serving_one_on_page(self):
        # ad-hoc solution for https://github.com/hhursev/recipe-scrapers/issues/527
        try:
            return (
                self.soup.find("div", {"data-test-id": "serving-amount-container"})
                .find("div", {"class": "fela-_txm046"})
                .select("div[class*=ds]")[0]
                .get_text()
                == "1"
            )
        except (AttributeError, IndexError):
            # AttributeError if .find(..) method errored
            # IndexError if the .select(..)[0] did not work as expected
            # both cases to fall back to default behaviour
            return True



from recipe_scrapers import EatTolerant
from tests import ScraperTest


class TestEatTolerant(ScraperTest):
    scraper_class = EatTolerant

    def test_host(self):
        self.assertEqual("eattolerant.de", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://eattolerant.de/suppe/vegane-rotkohlsuppe-mit-kokosmilch-histaminarm-glutenfrei-low-carb/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Vegane Rotkohlsuppe mit Kokosmilch"
        )

    def test_description(self):
        self.assertEqual(self.harvester_class.description(), "")

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://eattolerant.de/wp-content/uploads/2022/01/Vegane-Rotkohlsuppe-13-scaled.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "500 g Rotkohl (Nettogewicht)",
                "1 kleiner Apfel (z.B. Boskopp)",
                "200 g Kartoffeln",
                "1 Zwiebel",
                "1 kleines Stück Ingwer",
                "500 ml Gemüsebrühe",
                "200 ml Kokosmilch",
                "1 TL Zucker",
                "20 ml Verjus Sauer",
                "3 EL Kokosöl",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Die äußeren Blätter vom Rotkohl entfernen, anschließend halbieren, großzügig den Strunk entfernen und in kleine Stücke schneiden.\nDen Apfel sowie die Kartoffeln waschen und schälen, ebenfalls in kleine Stücke schneiden.\nDie Zwiebel in Würfel schneiden und den Ingwer fein reiben.\nDas Kokosöl in einem Top erhitzen, anschließend die Zwiebel und den Rotkohl kurz darin anbraten. Währenddessen mit Salz würzen.\nDie Apfel- und Kartoffelstücke zugeben, ebenfalls kurz anbraten.\nDie Gemüsebrühe, Kokosmilch und den Ingwer zugeben und die Suppe ca. 25 Minuten köcheln lassen. Kurz vor Ende der Garzeit den Verjus sowie Zucker zugeben. Anschließend fein pürieren und sofort servieren.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.atelierdeschefs import AtelierDesChefs
from tests import ScraperTest


class TestAtelierDesChefsScraper(ScraperTest):

    scraper_class = AtelierDesChefs

    def test_host(self):
        self.assertEqual("atelierdeschefs.fr", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("L'atelier des Chefs", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Cr\u00eape savoyarde")

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 Servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.atelierdeschefs.com/media/recette-e16689-crepe-savoyarde.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "Lait 1/2 \u00e9cr\u00e9m\u00e9 : 25 cl",
                "Beurre doux : 120 g",
                "Oeuf(s) : 1 pi\u00e8ce(s)",
                "Farine de bl\u00e9 : 70 g",
                "Farine de sarrasin (bl\u00e9 noir) : 55 g",
                "Sel fin : 8 pinc\u00e9e(s)",
                "Pomme(s) de terre \u00e0 chair ferme : 300 g",
                "Gros sel : 5 g",
                "Raclette au lait cru : 50 g",
                "Lardon(s) fum\u00e9(s) : 100 g",
                "Cr\u00e8me fra\u00eeche \u00e9paisse : 50 g",
                "Moulin \u00e0 poivre : 6 tour(s)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Pour la pâte à crêpes\nPour la pâte à crêpes : dans un bol, réunir les farines et le sel.\nCasser l’œuf et mélanger grossièrement, puis ajouter progressivement le lait tout en fouettant énergiquement.\nFaire fondre le beurre, puis l'ajouter à l'appareil.\nLaisser ensuite reposer au frais pendant 1 h.\nRéaliser les crêpes sur une crêpière chaude jusqu'à épuisement de la pâte.\nPour la garniture\nPréchauffer le four à 180 °C (th.\n6).\nÉplucher les pommes de terre et les couper en rondelles.\nLes disposer ensuite dans une casserole avec le gros sel, puis les couvrir d'eau.\nPorter à ébullition, puis cuire les pommes de terre pendant 5 min avant de les égoutter.\nDorer les lardons dans une poêle chaude, puis les égoutter sur une feuille de papier absorbant.\nCouper le fromage en fines tranches.\nDans une poêle chaude, faire fondre 10 g de beurre puis réchauffer doucement une crêpe.\nDisposer ensuite les rondelles de pommes de terre, puis les lardons.\nAjouter la crème et une tranche de fromage, saler et poivrer.\nEnfourner à 180 °C (th.\n6) jusqu'à ce que le fromage ait fondu.\nRenouveler l'opération pour les 5 autres crêpes.\nServir aussitôt.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(3.7, self.harvester_class.ratings())



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper
from ._utils import normalize_string


class Maangchi(AbstractScraper):
    @classmethod
    def host(cls):
        return "maangchi.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        before = self.soup.find("h4", string="Directions").find_all_previous("li")
        after = self.soup.find("h4", string="Ingredients:").find_all_next("li")
        list_before = [normalize_string(b.get_text()) for b in before]
        list_after = [normalize_string(a.get_text()) for a in after]
        list_before.reverse()
        return [x for x in list_before if x in list_after]

    def instructions(self):
        ols = self.soup.find("div", {"class": "entry"}).find_all("ol")
        instructions = []
        for ol in ols:
            for li in ol.findAll("li"):
                instructions.append(li)
        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )

    def ratings(self):
        return self.schema.ratings()

    def cuisine(self):
        return self.schema.cuisine()

    def description(self):
        return self.schema.description()



from recipe_scrapers.abril import Abril
from tests import ScraperTest


class TestAbrilScraper(ScraperTest):

    scraper_class = Abril

    def test_host(self):
        self.assertEqual("claudia.abril.com.br", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://claudia.abril.com.br/receitas/estrogonofe-de-carne/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Estrogonofe de carne")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "CLAUDIA")

    def test_image(self):
        self.assertEqual(
            "https://claudia.abril.com.br/wp-content/uploads/2020/02/receita-estrogonofe-de-carne.jpg?quality=85&strip=info&w=620&h=372&crop=1",
            self.harvester_class.image(),
        )

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "500 gramas de alcatra cortada em tirinhas",
                "1/4 xícara (chá) de manteiga",
                "1 unidade de cebola picada",
                "1 colher (sobremesa) de mostarda",
                "1 colher (sopa) de ketchup (ou catchup)",
                "1 pitada de pimenta-do-reino",
                "1 unidade de tomate sem pele picado",
                "1 xícara (chá) de cogumelo variado | variados escorridos",
                "1 lata de creme de leite",
                "• sal a gosto",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Derreta a manteiga e refogue a cebola até ficar transparente.\nJunte a carne e tempere com o sal.\nMexa até a carne dourar de todos os lados.\nAcrescente a mostarda, o catchup, a pimenta-do-reino e o tomate picado.\nCozinhe até formar um molho espesso.\nSe necessário, adicione água quente aos poucos.\nQuando o molho estiver encorpado e a carne macia, adicione os cogumelos e o creme de leite.\nMexa por 1 minuto e retire do fogo.\nSirva imediatamente, acompanhado de arroz e batata palha.\nDica: Se juntar água ao refogar a carne, frite-a até todo o líquido evaporar.""",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.kennymcgovern import KennyMcGovern
from tests import ScraperTest


class TestKennyMcGovernScraper(ScraperTest):

    scraper_class = KennyMcGovern

    def test_host(self):
        self.assertEqual("kennymcgovern.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://kennymcgovern.com/crispy-chicken-strips",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Crispy Chicken Strips")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Kenny McGovern")

    def test_image(self):
        self.assertEqual(
            self.harvester_class.image(),
            "https://kennymcgovern.com/wp-content/uploads/2020/02/crispy-chicken-strips-960x720.jpg",
        )

    def test_total_time(self):
        self.assertEqual(16, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("1 serving", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "8 tablespoons panko breadcrumbs",
                "4 tablespoons plain flour",
                "1/4 teaspoon garlic powder",
                "1/4 teaspoon onion granules",
                "1/4 teaspoon cayenne pepper",
                "1/4 teaspoon dried Italian herbs",
                "1/2 teaspoon sea salt",
                "1/4 teaspoon black pepper",
                "4 tablespoons plain flour",
                "1/4 teaspoon sea salt",
                "Pinch black pepper",
                "1 egg",
                "3 tablespoons milk",
                "1 skinless, boneless chicken breast fillet around 114g",
                "vegetable oil for deep frying",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "In a large bowl, add panko breadcrumbs, plain flour, garlic powder, onion granules, cayenne pepper, dried Italian herbs, sea salt & black pepper. Mix well.\nOn a plate, mix plain flour, sea salt & black pepper. Mix well.\nIn a bowl, combine egg and milk and whisk thoroughly.\nCut the chicken into 6-7 strips. Dip a chicken strip firstly into the plate of seasoned flour, then into the egg / milk mixture and finally into the breadcrumb mix, pressing well to ensure the chicken strip is nicely coated with breadcrumbs. Set the breaded chicken strip aside on a plate and repeat the process until all of the chicken strips are coated. At this stage the chicken strips are ready to cook, or can be covered and kept aside in the fridge for up to 24 hours.\nHeat oil for deep frying to around 180c. Carefully place each breaded chicken strip into the hot oil and fry for around 6 minutes, turning occasionally until the chicken strips are crispy and golden. Lift the chicken strips from the oil using a slotted spoon, drain off any excess oil and set aside on a plate.\nServe the crispy chicken strips with hot sauce, BBQ sauce or ranch dressing.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.eatingwell import EatingWell
from tests import ScraperTest


class TestEatingWell(ScraperTest):
    scraper_class = EatingWell

    def test_host(self):
        self.assertEqual("eatingwell.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Cheesy Ground Beef & Cauliflower Casserole"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Carolyn Casner")

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://imagesvc.meredithcorp.io/v3/mm/image?url=https%3A%2F%2Fstatic.onecms.io%2Fwp-content%2Fuploads%2Fsites%2F44%2F2021%2F08%2F16%2Fcheesy-ground-beef-and-cauliflower-casserole.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 tablespoon extra-virgin olive oil",
                "½ cup chopped onion",
                "1 medium green bell pepper, chopped",
                "1 pound lean ground beef",
                "3 cups bite-size cauliflower florets",
                "3 cloves garlic, minced",
                "2 tablespoons chili powder",
                "2 teaspoons ground cumin",
                "1 teaspoon dried oregano",
                "½ teaspoon salt",
                "¼ teaspoon ground chipotle",
                "1 (15 ounce) can no-salt-added petite-diced tomatoes",
                "2 cups shredded extra-sharp Cheddar cheese",
                "⅓ cup sliced pickled jalapeños",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            """Position rack in upper third of oven. Preheat broiler to high.
Heat oil in a large oven-safe skillet over medium heat. Add onion and bell pepper; cook, stirring, until softened, about 5 minutes. Add beef and cauliflower; cook, stirring and breaking the beef up into smaller pieces, until it is no longer pink, 5 to 7 minutes. Stir in garlic, chili powder, cumin, oregano, salt and chipotle; cook until fragrant, about 1 minute. Add tomatoes and their juices; bring to a simmer and cook, stirring occasionally, until liquid is reduced and the cauliflower is tender, about 3 minutes more. Remove from heat.
Sprinkle cheese over the beef mixture and top with sliced jalapeños. Broil until the cheese is melted and browned in spots, 2 to 3 minutes.""",
            self.harvester_class.instructions(),
        )

    def test_total_time(self):
        return self.assertEqual(30, self.harvester_class.total_time())



# mypy: allow-untyped-defs

from recipe_scrapers.leanandgreenrecipes import LeanAndGreenRecipes
from tests import ScraperTest


class TestLeanAndGreenRecipesScraper(ScraperTest):

    scraper_class = LeanAndGreenRecipes

    def test_host(self):
        self.assertEqual("leanandgreenrecipes.net", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("lean.green", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Yellow Squash Taco Casserole", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(40.0, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("5 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://leanandgreenrecipes.net/sites/default/files/2021-08/Yellow%20Squash%20Taco%20Casserole.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1.25 lbs Lean Ground Beef",
                "2 cup Yellow Squash",
                "1.5 tsp Cumin",
                "1.5 tsp Paprika",
                "1.5 tsp Chili Powder",
                "1/8 tsp Cayenne Pepper",
                "1/2 tsp Salt",
                "3/4 tsp Garlic Powder",
                "2 cup Low Fat Cheddar Cheese",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Preheat oven to 350*F.\nCook the ground beef. After browning the meat, add the canned Diced tomatoes and seasonings to the ground beef and simmer for 5 minutes.\nPlace the sliced squash to the bottom of a 9 x 13 inch baking pan.\nPlace meat mixture on top of squash, sprinkle grated cheese on top.\nBake in preheated oven for about 25 minutes.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(3.8, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("Mexican", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Tip: Recipe makes 5 servings, but casserole can be divided into individual 1 serving portions and freezing for later.",
            self.harvester_class.description(),
        )



# mypy: disallow_untyped_defs=False
from urllib.parse import parse_qs, urlparse

import requests

from ._abstract import AbstractScraper

HEADERS = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:86.0) Gecko/20100101 Firefox/86.0"
}
# These languages use measurements like cups and spoons rather than grams and liters
IMPERIAL_LANGUAGES = ["en"]
KPTN_DEFAULT_LANGUAGE = "en"


class KptnCook(AbstractScraper):
    def __init__(self, url, proxies=None, timeout=None, *args, **kwargs):
        super().__init__(url, proxies=proxies, timeout=timeout, *args, **kwargs)
        parsed_url = urlparse(self.url)
        # Extract language from URL
        query = parse_qs(parsed_url.query)
        self.lang = query["lang"][0] if "lang" in query else KPTN_DEFAULT_LANGUAGE

        # Build final recipe url (of type mobile.kptncook.com/*)
        self.final_url = "".join(
            ["https://", parsed_url.hostname, parsed_url.path, f"?lang={self.lang}"]
        )
        # Extract recipe id from the url path
        recipe_uid = parsed_url.path.split("/")[-1]

        # Request the final recipe json from the kptncook api
        api_url = f"https://mobile.kptncook.com/recipes/search?kptnkey=6q7QNKy-oIgk-IMuWisJ-jfN7s6&lang={self.lang}"
        json_request_body = [{"uid": recipe_uid}]
        self.recipe_json = requests.post(
            api_url,
            headers=HEADERS,
            proxies=proxies,
            timeout=timeout,
            json=json_request_body,
        ).json()[0]

    @classmethod
    def host(cls, subdomain="mobile"):
        return f"{subdomain}.kptncook.com"

    def author(self):
        author = self.recipe_json["authors"][0]
        return f"{author['name']} ({author['link']})"

    def title(self):
        return self.recipe_json["title"]

    def category(self):
        return self.recipe_json["rtype"]

    def total_time(self):
        return self.recipe_json["cookingTime"] + self.recipe_json["preparationTime"]

    def cook_time(self):
        return self.recipe_json["cookingTime"]

    def prep_time(self):
        return self.recipe_json["preparationTime"]

    def yields(self):
        return 1

    def nutrients(self):
        return self.recipe_json["recipeNutrition"]

    def canonical_url(self):
        return self.final_url

    def image(self):
        return f"{self.recipe_json['imageList'][0]['url']}?kptnkey=6q7QNKy-oIgk-IMuWisJ-jfN7s6"

    def ingredients(self):
        return [
            " ".join(
                str(x)
                for x in
                # The filter is needed because "measure" and "quantity" fields are not always provided
                filter(
                    None,
                    [
                        ingredient.get("quantity"),
                        ingredient.get("measure"),
                        ingredient["ingredient"]["title"],
                    ]
                    if self.lang not in IMPERIAL_LANGUAGES
                    else [
                        ingredient.get("quantityUS"),
                        ingredient.get("measureUS"),
                        ingredient["ingredient"]["title"],
                    ],
                )
            )
            for ingredient in self.recipe_json["ingredients"]
        ]

    def instructions(self):
        return "\n".join(step["title"] for step in self.recipe_json["steps"])

    def ratings(self):
        return None

    def cuisine(self):
        return None

    def description(self):
        return self.recipe_json["authorComment"]

    def language(self):
        return self.lang



from recipe_scrapers.eatsmarter import Eatsmarter
from tests import ScraperTest


class TestEatSmarter(ScraperTest):

    scraper_class = Eatsmarter

    def test_host(self):
        self.assertEqual("eatsmarter.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://eatsmarter.com/recipes/citrus-avocado-salad-with-almonds",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Citrus Avocado Salad with Almonds"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "EAT SMARTER")

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 Tbsps slivered almonds",
                "2 stalks Celery",
                "2 ozs green Olives (pitted)",
                "3 Oranges",
                "1 pink Grapefruit",
                "1 Endive",
                "1 Avocado",
                "2 Tbsps olive oil",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Toast almonds in a dry pan until golden and fragrant. Place into a bowl to cool. Rinse and thinly slice celery. Cut olives into slices. Squeeze orange juice from one orange. Peel remaining oranges well and fillet, cut fillets into pieces. Repeat the same steps with grapefruit. Separate endive into individual leaves, rinse and spin dry, cut into strips. Peel and pit avocado, cut pulp into small cubes. Combine avocado cubes with orange juice in a bowl. Add all remaining prepared salad ingredients, drizzle with oil and toss salad carefully. Arrange on plates and serve.",
            self.harvester_class.instructions(),
        )

    def test_total_time(self):
        self.assertEqual(20, self.harvester_class.total_time())

    def test_cook_time(self):
        return self.assertEqual(None, self.harvester_class.cook_time())

    def test_prep_time(self):
        return self.assertEqual(None, self.harvester_class.prep_time())



from recipe_scrapers.cybercook import Cybercook
from tests import ScraperTest


class TestCybercook(ScraperTest):

    scraper_class = Cybercook

    def test_host(self):
        self.assertEqual("cybercook.com.br", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://cybercook.com.br/receitas/aves/strogonoff-de-frango-16644",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Strogonoff de Frango")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Leticia Obo Andreghetti")

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://img.cybercook.com.br/imagens/receitas/644/strogonoff-de-frango-1.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "Molho de tomate 200 gramas",
                "Cebola 1 unidade",
                "Alho 2 dentes",
                "Creme de Leite 1 lata",
                "Peito de Frango 600 gramas",
                "Óleo de soja 2 colheres (sopa)",
                "Sal a gosto",
                "Pimenta-do-Reino Branca a gosto",
                "Champignon em conserva 100 gramas",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Primeiro corte o frango em cubinhos.\nEm uma panela média, coloque o óleo, o alho e a cebola e espere dourar.\nDepois coloque o frango e o sal e a pimenta a gosto, aqueça até o ponto de fritura.\nMexa bem e tampe meia panela para que crie água, espere.\nSumir a água e começar a fritura.\nQuando o frango já tiver dourado, acrescente o molho de tomate e o champignon.\nDepois coloque a lata de creme de leite e mexa até espalhar, com a mesma lata encha de água.\nMexa mais uma vez até misturar e deixe levantar fervura.\nO strogonoff está pronto para ser servido.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.0, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
import json
import re

from ._abstract import AbstractScraper
from ._schemaorg import SchemaOrg


class MobKitchen(AbstractScraper):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        recipe_schema = None
        for schema in self.soup.head.find_all("script", type="application/ld+json"):
            recipe_schema = schema.find(string=re.compile('"@type":"Recipe"'))

            if recipe_schema:
                break

        recipe_schema = json.loads(recipe_schema)
        self.schema = SchemaOrg(recipe_schema, raw=True)

    @classmethod
    def host(cls, domain="mobkitchen.co.uk"):
        return domain

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def cuisine(self):
        return self.schema.cuisine()

    def description(self):
        return self.schema.description()



from recipe_scrapers.reishunger import Reishunger
from tests import ScraperTest


class TestReishungerScraper(ScraperTest):
    scraper_class = Reishunger
    test_file_name = "reishunger_2"

    def test_host(self):
        self.assertEqual("reishunger.de", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("snackicat", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Süßkartoffel-Kichererbsen-Curry", self.harvester_class.title()
        )

    def test_total_time(self):
        self.assertEqual(15, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("3 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cdn.reishunger.com/img7830bearbeitetklein.jpg?quality=85",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "240g Basmati Reis Pusa",
                "1 TL Kokosöl",
                "1 Dose Bio Kichererbsen",
                "200ml Kokosmilch",
                "1 Zwiebel",
                "1 Knoblauchzehe",
                "1 mittelgroße Süßkartoffel (hier: 300g)",
                "1 Paprika",
                "150ml Wasser",
                "20g Ingwer",
                "3 TL Curry",
                "Salz, Cayenne Pfeffer, Paprika edelsüß",
                "Kräuter nach Belieben (hier: Petersilie)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "Zubereitung im Kochtopf Reis waschen. Den Reis mit kaltem Wasser bedecken. Mit den Händen den Reis in kreisenden Bewegungen waschen. Durch die überschüssige Stärke wird das Wasser trüb. Nun das Wasser abgießen und den Vorgang wiederholen bis das Wasser klar bleibt. Reis in einen Kochtopf geben. Wasser dazugeben. Nach Belieben salzen. Reis 10 Minuten einweichen lassen. Herd auf die höchste Hitzestufe stellen und Reis aufkochen lassen. Sobald das Wasser kocht, den Herd auf die mittlere Hitzestufe stellen und den Reis ca. 15 Minuten bei geschlossenem Deckel köcheln lassen bis das Wasser komplett aufgesogen wurde. Nach Belieben ein Stück Butter hinzufügen.",
                    'Zubereitung im Reiskocher Reis waschen. Den Reis mit kaltem Wasser bedecken. Mit den Händen den Reis in kreisenden Bewegungen waschen. Durch die überschüssige Stärke wird das Wasser trüb. Nun das Wasser abgießen und den Vorgang wiederholen bis das Wasser klar bleibt. Anschließend den Reis in den Innentopf geben. Reis in den Reiskocher geben. Wasser dazugeben. Nach Belieben salzen. Deckel schließen und den Kochvorgang im Modus "Weiß" starten. Sobald der Reiskocher in den Warmhaltemodus schaltet, ist der Reis fertig. Nach Belieben ein Stück Butter hinzufügen.',
                    "Zubereitung im Mikrowellen-Reiskocher Reis waschen. Den Reis mit kaltem Wasser bedecken. Mit den Händen den Reis in kreisenden Bewegungen waschen. Durch die überschüssige Stärke wird das Wasser trüb. Nun das Wasser abgießen und den Vorgang wiederholen bis das Wasser klar bleibt. Reis in den Mikrowellen Reiskocher geben. Wasser dazugeben. Nach Belieben salzen. Deckel aufsetzen und bei höchster Stufe (600-800 Watt) für 11 Minuten in die Mikrowelle stellen. Nach Belieben ein Stück Butter hinzufügen.",
                    "Zubereitung im Dämpfer Reis waschen. Du bedeckst den Reis mit kaltem Wasser. Mit den Händen bearbeitest du den Reis in kreisenden Bewegungen. Durch die überschüssige Stärke wird das Wasser trüb. Nun gießt du das Wasser ab und wiederholst den Vorgang solange bis das Wasser klar bleibt. Bambuskorb mit einem Baumwolltuch auslegen und den Reis darauf geben. Mit Stäbchen oder Gabel Löcher in die Masse drücken durch die Dampf entweichen kann. Bambuskorb mit Korbdeckel schließen. Topfboden mit ca. 1 cm Wasser bedecken und Bambuskorb reinstellen. Bei niedriger bis mittlerer Hitze ca. 20 Minuten dämpfen lassen. Zwischenzeitlich Wasser nachkippen, sodass der Korb stetig in ca. 1 cm hohem Wasser steht. Reis entnehmen und nach Belieben ein Stück Butter hinzufügen.",
                    "Zwiebeln und Knoblauch klein schneiden und in Kokosöl in einer Pfanne kurz anbraten.",
                    "Kokosmilch und Wasser vorsichtig in die Pfanne schütten.",
                    "Süßkartoffel und Paprika klein schneiden und in die Pfanne geben.",
                    "Kichererbsen abtropfen, Ingwer schälen und kleinschneiden, ebenfalls in die Pfanne geben.",
                    "Curry und Salz hinzugeben und bei geschlossenem Deckel kurz zum Kochen bringen, dann auf leichter Stufe circa 10 Minuten köcheln lassen.",
                    "Paprika edelsüß und Cayennepfeffer hinzugeben, verrühren und ca. weitere 5 Minuten köcheln lassen, bis die Soße schön eingekocht ist.",
                    "Mit Reis und Kräutern servieren, fertig. Guten Reishunger!",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5, self.harvester_class.ratings())



from recipe_scrapers.lovingitvegan import Lovingitvegan
from tests import ScraperTest


class TestLovingitveganScraper(ScraperTest):

    scraper_class = Lovingitvegan

    def test_host(self):
        self.assertEqual("lovingitvegan.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://lovingitvegan.com/kale-smoothie/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Kale Smoothie")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Alison Andrews")

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://lovingitvegan.com/wp-content/uploads/2018/08/Kale-Smoothie-8.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 1/4 cups Soy Milk ((300ml) or Almond Milk)",
                "2 Frozen Bananas ((200g) previously peeled, broken into quarters and frozen for at least 12 hours)",
                "1/2 cup Raw Cashews ((75g))",
                "2 cups Kale ((56g) Torn up, packed)",
                "4 Medjool Dates (Pitted)",
                "1 tsp Minced Ginger",
                "1/8 tsp Cinnamon",
                "1 Tbsp Lime Juice (freshly squeezed)",
                "1 cup Ice Cubes",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Add the soy milk to your blender jug and then the frozen bananas, cashews, kale, dates, minced ginger, cinnamon and fresh lime juice. Top with ice cubes.\nBlend until very smooth.\nPour out into glasses and serve immediately.",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_yields, normalize_string


class BigOven(AbstractScraper):
    @classmethod
    def host(cls):
        return "bigoven.com"

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return get_yields(self.soup.find("div", {"class": "yield"}).text)

    def image(self):
        return self.schema.image()

    def ingredients(self):
        rows = self.soup.find("ul", {"class": "ingredients-list"}).findAll("li")
        return [
            normalize_string(row.span.text)
            for row in rows
            if "ingHeading" not in row.span["class"]
        ]

    def instructions(self):
        ps = self.soup.find("div", {"class": "instructions"}).findAll("p")
        return "\n".join([normalize_string(p.text) for p in ps])

    def ratings(self):
        try:
            cnt = (
                self.soup.find("div", {"class": "recipe-rating"})
                .find("span", {"class": "count"})
                .text
            )
            rating = (
                self.soup.find("div", {"class": "recipe-rating"})
                .find("span", {"class": "rating"})
                .text
            )
            return {"count": int(cnt), "rating": round(float(rating), 2)}
        except Exception:
            return None



# mypy: allow-untyped-defs

from recipe_scrapers.omnivorescookbook import OmnivoresCookbook
from tests import ScraperTest


class TestOmnivoresCookbookScraper(ScraperTest):

    scraper_class = OmnivoresCookbook

    def test_host(self):
        self.assertEqual("omnivorescookbook.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Maggie Zhu", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Stir Fried Bok Choy with Tofu Puffs", self.harvester_class.title()
        )

    def test_category(self):
        self.assertEqual("Side", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(15, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://omnivorescookbook.com/wp-content/uploads/2022/02/211213_Stir-Fried-Bok-Choy-With-Tofu-Puffs_550.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 lb (450 g) bok choy , chopped to bite size pieces (see the cutting method in the post above)",
                "1 heaping cup (60 grams) tofu puffs , halved",
                "2 green onions , sliced",
                "1 tablespoon peanut oil (or vegetable oil)",
                "1 teaspoon sugar",
                "2 tablespoons light soy sauce (or soy sauce)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Heat 1 tablespoon of oil in a medium-sized wok or large skillet over high heat until hot. Add scallion and stir a few times until fragrant.\nAdd bok choy. Stir and cook for 1 to 2 minutes, to coat evenly with oil.\nSprinkle it with sugar and swirl in light soy sauce. Immediately stir a few times to mix well. Add deep fried tofu, then stir again for about 20 seconds.\nCover the wok and lower to medium-low heat. Steam for 30 seconds. Uncover and stir to check the doneness. Cover and cook for another 10 to 20 seconds again, if necessary, until the bok choy is cooked through and slightly caramelized on the edges. Turn off the heat and transfer everything to a serving plate.\nServe hot as a side dish or over steamed rice as a light main dish.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("Chinese", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "A super quick and easy stir fried bok choy with tofu puffs that only uses five ingredients and takes 15 minutes to prep and cook.",
            self.harvester_class.description(),
        )



from recipe_scrapers.fredriksfikaallas import FredriksFikaAllas
from tests import ScraperTest


class TestFredriksFikaAllasScraper(ScraperTest):

    scraper_class = FredriksFikaAllas

    def test_host(self):
        self.assertEqual("fredriksfika.allas.se", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://fredriksfika.allas.se/fredriks-brod/mormors-basta-tekakor/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual("Mormors bästa tekakor", self.harvester_class.title())

    def test_category(self):
        self.assertEqual("Fredriks bröd", self.harvester_class.category())

    def test_image(self):
        self.assertEqual(
            "https://files-aller-blogger-platform.aws.aller.com/uploads/sites/87/2015/08/10072015-_MG_0917-860x573.jpg?fit=crop&h=630&w=1200&ar=1.91:1",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 dl havregryn",
                "5 dl vatten",
                "50 g smör",
                "0,5 dl ljus sirap",
                "2 tsk salt",
                "50 g jäst",
                "9-12 dl rågsikt",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "1. Koka en gröt på havregryn och 2,5 dl vatten.\n2. Smält smöret i gröten, samt blanda i resten av vattnet, sirapen och saltet.\n3. Smula jästen i en bunke, ha i grötblandningen och rör tills jästen löst upp sig.\n4.\n Blanda i mjölet och arbeta degen ihop till en deg, mjölmängden varierar\n på hur länge man kokat gröten så därav kan det variera. Degen ska gå\natt ta på utan att den klibbar på fingrarna.\n5. Låt jäsa 30 minuter.\n6.\n Ha upp degen på ett bakbord och knåda den lite. Kavla ut degen ca 0,5\ncm hög. Stansa ut rundlar och lägg på bakplåtspapperklädd plåt. Nagga\ntekakorna med en gaffel.\n7. Låt jäsa under en handduk ca 30 minuter. Sätt ugnen på 225 grader.\n8. Baka mitt i ugnen i 6-8 minuter tills de får en gyllene färg.",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
import re

from bs4 import Tag

from ._abstract import AbstractScraper
from ._utils import normalize_string

"""
    NOTE: This website has at least 2 prominent layouts styles, so there are two logic blocks and 2 test cases to
    support in ingredients and instructions processing sections.
"""


class FarmhouseDelivery(AbstractScraper):
    @classmethod
    def host(cls, domain="com"):
        return f"recipes.farmhousedelivery.{domain}"

    def title(self):
        return self.soup.find("h1", {"class": "entry-title"}).get_text(strip=True)

    def ingredients(self):
        # Style 1
        ingredients_marker = self.soup.find("p", string=re.compile(r"Ingredients:"))
        if ingredients_marker is not None:
            ingredients_marker_siblings = ingredients_marker.next_siblings
            for ingredients_marker_sibling in ingredients_marker_siblings:
                if (
                    isinstance(ingredients_marker_sibling, Tag)
                    and ingredients_marker_sibling.name == "ul"
                ):
                    ingredients = ingredients_marker_sibling.findAll("li")
                    return [
                        normalize_string(ingredient.get_text())
                        for ingredient in ingredients
                    ]

        # Style 2
        ingredients_marker = self.soup.find("p", string=re.compile(r"Ingredients"))
        if ingredients_marker is not None:
            ingredients = []
            ingredients_marker_siblings = ingredients_marker.next_siblings
            for ingredients_marker_sibling in ingredients_marker_siblings:
                if (
                    isinstance(ingredients_marker_sibling, Tag)
                    and ingredients_marker_sibling.name == "p"
                ):
                    if ingredients_marker_sibling.get_text() == "Instructions":
                        break
                    else:
                        ingredients.append(
                            normalize_string(ingredients_marker_sibling.get_text())
                        )
            return ingredients

        return None

    def _instructions_list(self):
        # Style 1
        instructions_marker = self.soup.find("p", string=re.compile(r"Instructions:"))
        if instructions_marker is not None:
            instructions_marker_siblings = instructions_marker.next_siblings
            for instructions_marker_sibling in instructions_marker_siblings:
                if (
                    isinstance(instructions_marker_sibling, Tag)
                    and instructions_marker_sibling.name == "p"
                    and instructions_marker_sibling.get_text(strip=True) != ""
                ):
                    instructions = instructions_marker_sibling.findAll("span")
                    return [
                        normalize_string(instruction.get_text())
                        for instruction in instructions
                    ]

        # Style 2
        instructions_marker = self.soup.find("p", string=re.compile(r"Instructions"))
        if instructions_marker is not None:
            instructions = []
            instructions_marker_siblings = instructions_marker.next_siblings
            for instructions_marker_sibling in instructions_marker_siblings:
                if (
                    isinstance(instructions_marker_sibling, Tag)
                    and instructions_marker_sibling.name == "p"
                    and instructions_marker_sibling.get_text(strip=True) != ""
                ):
                    instructions.append(
                        normalize_string(instructions_marker_sibling.get_text())
                    )
            return instructions

        return None

    def instructions(self):
        data = self._instructions_list()
        return "\n".join(data) if data else None

    def image(self):
        container = self.soup.find("div", {"class": "entry-content"})
        if not container:
            return None

        image = container.find("img", {"src": True})
        return image["src"] if image else None



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class Przepisy(AbstractScraper):
    @classmethod
    def host(cls):
        return "przepisy.pl"

    def title(self):
        return self.soup.find("h1", {"class": "title"}).get_text()

    def author(self):
        return self.schema.author()

    def total_time(self):
        return get_minutes(self.soup.find("div", {"class": "time-count"}))

    def yields(self):
        return get_yields(self.soup.find("div", {"class": "person-count"}))

    def ingredients(self):
        ingredients = self.soup.findAll("span", {"class": "text-bg-white"})

        return [
            normalize_string(i.get_text()) + " " + normalize_string(j.get_text())
            for i, j in zip(ingredients[0::2], ingredients[1::2])
        ]

    def instructions(self):
        instructions = self.soup.findAll("p", {"class": "step-info-description"})

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )



from recipe_scrapers.thenutritiouskitchen import TheNutritiousKitchen
from tests import ScraperTest


class TestTheNutritiousKitchenScraper(ScraperTest):

    scraper_class = TheNutritiousKitchen

    def test_host(self):
        self.assertEqual("thenutritiouskitchen.co", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("The Nutritious Kitchen", self.harvester_class.author())

    def test_canonical_url(self):
        self.assertEqual(
            "https://thenutritiouskitchen.co/healthy-fig-pecan-crumble-bars/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Healthy Fig Pecan Crumble Bars")

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("12 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://thenutritiouskitchen.co/wp-content/uploads/2020/09/figpecanbars-10-of-17-225x225.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 cups finely ground almond flour",
                "3 TBSP cassava flour or coconut flour",
                "Cinnamon + Sea salt to taste",
                "1/2 cup pecan butter or cashew butter (runny)",
                "1/4 cup melted coconut oil",
                "1/4 cup maple syrup",
                "16 oz. fresh figs",
                "Optional: 1 tbsp. maple syrup",
                "Handful of chopped pecans for topping",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat oven to 350 degrees Fahrenheit. Line an 8×8 baking tray with parchment paper and set aside.\nIn a large bowl whisk together almond flour, cassava, sea salt + cinnamon. In a separate bowl whisk pecan butter, melted coconut oil + maple syrup until a thick glaze forms.\nPour into dry ingredients and mix together with a frosting spatula until dough forms. Press into baking tray, reserving about 1/4 cup for the topping. Use hands to spread out if needed.\nIn a microwave-safe bowl, heat up figs for about 1 minute then crush with a fork to make quick “jam”. Mix in optional tbsp of maple syrup.\nLayer jam then the remaining dough into “crumbles” Top with pecans.\nBake for about 20-25 minutes depending on your oven. Allow to cool in pan then cool completely on a rack before slicing into 12 squares.\nStore covered in the fridge up to 5 days. They are great served chilled or warm!",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.cookeatshare import CookEatShare
from tests import ScraperTest


class TestCookEatShare(ScraperTest):

    scraper_class = CookEatShare

    def test_host(self):
        self.assertEqual("cookeatshare.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual("Pork Steak Vegetable Bake", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "CookEatShare Cookbook")

    def test_image(self):
        self.assertEqual(
            "https://assets.cookeatshare.com/assets/recipe-art/stock/3/full-f1e2882559f1146065432f8bdd182440.png",
            self.harvester_class.image(),
        )

    def test_total_time(self):
        self.assertEqual(None, self.harvester_class.total_time())

    def test_ingredients(self):
        self.assertEqual(
            [
                '4 med. potatoes, pared & cut lengthwise, in 1/4" slices',
                "1 lg. carrot, pared & sliced",
                '4 pork steaks, cut 1/2" thick',
                "1/2 c. water",
                "1 env. dry onion soup mix or possibly 1 can cream of onion soup",
                "2 tbsp. soy sauce",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            'Place potatoes and carrots in bottom of 11" x 7 1/2" x 1 3/4" baking dish. Trim fat from steaks. In large skillet, cook trimmings till about 2 Tbsp. oil accumulates; throw away trimmings.\nIn warm oil, brown steaks well on both sides. In small saucepan, bring to boil. Spoon 1/2 of onion soup mix over the potatoes and carrots; top with steaks. Spoon remaining soup mix over. Cover, bake in 350 degree oven for 1 hour. Uncover and bake 10 min more. Makes 4 servings.',
            self.harvester_class.instructions(),
        )



from recipe_scrapers.handletheheat import HandleTheHeat
from tests import ScraperTest


class TestHandleTheHeatScraper(ScraperTest):

    scraper_class = HandleTheHeat

    def test_host(self):
        self.assertEqual("handletheheat.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://handletheheat.com/chewy-brownies/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Best Ever Chewy Brownies")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Tessa Arias")

    def test_yields(self):
        self.assertEqual("9 servings", self.harvester_class.yields())

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_image(self):
        self.assertEqual(
            "https://handletheheat.com/wp-content/uploads/2017/03/Chewy-Brownies-Square-1.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "5 tablespoons (71 grams) unsalted butter",
                "1 1/4 cups (249 grams) granulated sugar",
                "2 large eggs plus 1 egg yolk, (cold)",
                "1 teaspoon vanilla extract",
                "1/3 cup vegetable oil",
                "3/4 cup (75 grams) unsweetened cocoa powder",
                "1/2 cup (63 grams) all-purpose flour",
                "1/8 teaspoon baking soda",
                "1 tablespoon cornstarch",
                "1/4 teaspoon salt",
                "3/4 cup (128 grams) semisweet chocolate chips",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat the oven to 325°F. Line a 8 by 8-inch pan with foil or parchment paper and spray with nonstick cooking spray.\nIn a microwave safe bowl, add the butter and sugar. Microwave for about 1 minute, or until the butter is melted. Whisk in the eggs, egg yolk, and vanilla. Stir in the oil and cocoa powder.\nWith a rubber spatula, stir in the flour, baking soda, cornstarch, and salt until combined. Stir in the chocolate chips.\nSpread the brownie batter evenly into the prepared pan. Place in the oven and bake for 30 minutes, or until the brownies are set and a cake tester inserted into the center has moist crumbs attached. Do not overcook. Let cool completely before cutting and serving.\nBrownies can be stored in an airtight container at room temperature for up to 3 days.",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes


class MarthaStewart(AbstractScraper):
    @classmethod
    def host(cls):
        return "marthastewart.com"

    def title(self):
        return self.schema.title()

    def total_time(self):
        s = (
            self.soup.findAll("div", {"class": "two-subcol-content-wrapper"})[0]
            .find("div", {"class": "recipe-meta-item-body"})
            .text.strip()
        )
        return get_minutes(s)

    def yields(self):
        return (
            self.soup.findAll("div", {"class": "two-subcol-content-wrapper"})[1]
            .find("div", {"class": "recipe-meta-item-body"})
            .text.strip()
        )

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()



from recipe_scrapers.redhousespice import RedHouseSpice
from tests import ScraperTest


class TestRedHouseSpiceScraper(ScraperTest):

    scraper_class = RedHouseSpice

    def test_host(self):
        self.assertEqual("redhousespice.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Wei Guo", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Easy Char Siu (Chinese BBQ pork, 叉烧)", self.harvester_class.title()
        )

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("3 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://redhousespice.com/wp-content/uploads/2020/05/Char-siu-Chinese-BBQ-pork-14-scaled.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 pork shoulder steaks (aka pork butt) (about 350g/12oz, see note 1)",
                "4 tbsp Char Siu sauce (see note 2)",
                "1 tbsp oyster sauce",
                "1/2 tbsp light soy sauce",
                "1/4 tsp Chinese five-spice powder",
                "4 cloves garlic, finely sliced",
                "5 slices ginger",
                "1/2 tsp chilli powder (optional, see note 4)",
                "2 tsp honey",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Marinate the meat\nPut pork steaks in a resealable plastic bag. Add all the ingredients for the marinade.\nSqueeze out air then seal the bag. Rub around for an even coating. Store in the fridge for at least 6 hours (ideally overnight).\nTake the meat out of the bag right before roasting. Keep the marinade for later use.\nPrepare for roasting\nPreheat the oven at 425°F/220°C/Fan 200°C.\nIf using a baking tray with a wire rack that fits inside, fill the tray with hot water (lower than the rack) and put the steak on the rack. Place the tray in the middle of the oven.\nAlternatively, place a large tray with hot water at the bottom of the oven. Then place the steak on the middle rack of the oven.\nRoast & brush (see note 5)\nLeave the meat to roast for 15 mins. Take out and flip it over. Brush some marinade then put back into the oven (Make sure there is always enough water in the tray).\nCook for a further 10 mins. While waiting, mix 2 teaspoons of honey with 2 teaspoons of the marinade.\nThen increase the oven temperature to 460°F/240°C/Fan 220°C. Take out the meat. Brush with the honey mixture.\nPut back into the oven for 5 mins. Then brush the other side with the honey mixture. Roast for a final 3 mins.\nServe\nLeave the meat to rest for 5 mins then slice and serve it in your preferred way.\nYou may also heat up the remaining marinade (remove the garlic & ginger) then serve it as a sauce, a soup base, or a noodle seasoning, etc.""",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



from recipe_scrapers.averiecooks import AverieCooks
from tests import ScraperTest


class TestAverieCooksScraper(ScraperTest):

    scraper_class = AverieCooks

    def test_host(self):
        self.assertEqual("averiecooks.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.averiecooks.com/balsamic-watermelon-and-cucumber-salad/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Balsamic Watermelon Cucumber Salad"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Averie Sunshine")

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.averiecooks.com/wp-content/uploads/2020/07/watermelonsalad-5-480x480.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "500 millilters balsamic vinegar",
                "1 cup granulated sugar, divided",
                "3 cups watermelon, seeded and cubed (I recommend seedless, firm watermelon)",
                "1 large cucumber or English cucumber, peeled and cubed",
                "1 cup argula (1 heaping handful)",
                "1/3 cup goat cheese, crumbled or as desired",
                "1/3 cup candied nuts, or as desired",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "To a high-sided medium/large kettle (use one bigger than you think you need), add the vinegar, 1/2 cup sugar, and heat over medium to medium-high until mixture boils and can sustain a fast rolling boil.\nBoil for about 15 to 20 minutes, or until reduced by about 80% and has thickened and is syrupy; stir intermittently and keep an eye on it so it doesn't bubble over.\nWhen the sauce looks like it's about halfway done, taste the sauce, and if it's too vinegary and bitter for you, add part of or all of the remaining sugar. I personally use almost 1 cup. Sauce will thicken up more as it cools.\nTo a medium bowl, add all the remaining ingredients, stir to combine, and drizzle as much of the balsamic reduction as desired.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.greatbritishchefs import GreatBritishChefs
from tests import ScraperTest


class TestGreatBritishChefsScraper(ScraperTest):

    scraper_class = GreatBritishChefs

    def test_host(self):
        self.assertEqual("greatbritishchefs.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.greatbritishchefs.com/recipes/picadillo-recipe",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Picadillo")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Great British Chefs")

    def test_total_time(self):
        self.assertEqual(50, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://media-cdn.greatbritishchefs.com/media/mwtkdmv3/img73858.jpg?mode=crop&width=768&height=512",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "500g of beef mince",
                "1 onion, diced",
                "2 garlic cloves, sliced",
                "1 carrot, finely diced",
                "1 potato, finely diced",
                "1 bay leaf",
                "1 tsp ground cumin",
                "1 tsp ground coriander",
                "1 tsp smoked paprika",
                "2 tbsp of tomato purée",
                "100g of raisins",
                "300ml of chicken stock",
                "300g of tomatillo salsa",
                "1 tsp honey",
                "lime juice, to taste",
                "1 handful of coriander, chopped",
                "salt, to taste",
                "olive oil",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "To begin, heat a splash of olive oil in a large pan and add the mince. Cook until nicely browned – you may need to do this in batches to avoid overcrowding the mince\nDecant the mince into a bowl and sweat the onions, garlic, carrot and potato in the same pan you used to brown the mince. Add the bay leaf and spices\nOnce the onions are translucent, return the mince to the pan and add the tomato purée, raisins, chicken stock and tomatillo salsa. Turn down the heat and simmer for 30 minutes\nSeason with the honey, lime juice and salt. Stir through the coriander and serve with rice or as a taco filling",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            "This simple picadillo recipe has the perfect balance of sweetness and spice. A pleasing tanginess comes from the addition of vibrant tomatillo salsa for a great depth of flavour.",
            self.harvester_class.description(),
        )



from recipe_scrapers.thekitchenmagpie import TheKitchenMagPie
from tests import ScraperTest


class TestTheKitchenMagPie(ScraperTest):
    scraper_class = TheKitchenMagPie

    def test_host(self):
        self.assertEqual("thekitchenmagpie.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.thekitchenmagpie.com/salmon-loaf/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual("Salmon Loaf", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Karlynn Johnston")

    def test_total_time(self):
        self.assertEqual(70, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "16 ounces of canned salmon (drained)",
                "1 cup seasoned fine breadcrumbs",
                "½ cup onion (finely chopped)",
                "1 tsp dill weed",
                "2 large eggs (beaten)",
                "1 tablespoon lemon juice",
                "½ tsp salt",
                "¼ tsp black pepper",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat your oven to 375 °F. Grease a baking sheet and set aside.\nDrain the salmon well and if desired, remove any skin and bones from the salmon. You can leave them in, some people love those the best! They will bake right in, don't worry.\nFlake the salmon with a fork and then mix with the breadcrumbs, onion, dill, egg, lemon juice, salt and pepper.\nWith clean hands, shape into a loaf on the greased baking sheet.\nBake for 40-50 minutes OR until nicely browned AND reaches an internal temperature of at least 165 °F.\nLet cool for 5 minutes, then slice and serve. Garnish with fresh parsley and lemon wedges. This is excellent with lemon juice squeezed on top!",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.realsimple import RealSimple
from tests import ScraperTest


class TestRealSimpleScraper(ScraperTest):

    scraper_class = RealSimple

    def test_host(self):
        self.assertEqual("realsimple.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.realsimple.com/food-recipes/browse-all-recipes/classic-cheesecake-recipe",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Classic Cheesecake")

    def test_image(self):
        self.assertEqual(
            "https://www.realsimple.com/thmb/EFNZU3tZG_O0FvomS1ExHzse4qI=/300x300/smart/filters:no_upscale()/classic-cheesecake_300-70617627cf5f4f5eae7f1a11018713ec.jpg",
            self.harvester_class.image(),
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Dawn Perry")

    def test_total_time(self):
        self.assertEqual(270, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("10 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "Cake:",
                "18 graham crackers (2 sleeves)",
                "0.5 cup (1 stick) unsalted butter, melted",
                "0.25 teaspoon kosher salt",
                "1 cup plus 4 tablespoons sugar",
                "3 8-ounce packages cream cheese, at room temperature",
                "2 cups sour cream, at room temperature",
                "1.5 teaspoons pure vanilla extract",
                "3 large eggs, at room temperature",
                "Cherry sauce:",
                "1 10-ounce bag frozen cherries",
                "0.5 cup sugar",
                "0.25 teaspoon kosher salt",
                "1 tablespoons cornstarch",
                "2 tablespoons fresh lemon juice",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Make the cake: Heat oven to 325° F. In a food processor, pulse the graham crackers until fine crumbs form. Add the butter, salt, and 2 tablespoons of the sugar and pulse to combine. Using a straight-sided dry measuring cup, press the mixture into the bottom and 2 inches up the sides of a 9-inch springform pan.\nUsing an electric mixer, beat the cream cheese and 1 cup of the remaining sugar on medium speed until smooth. Add 1 cup of the sour cream and 1 teaspoon of the vanilla and beat to combine. Beat in the eggs one at a time. Pour the mixture into the crust and bake until just set (the center will be slightly wobbly), 50 to 60 minutes.\nIn a small bowl, combine the remaining 1 cup of sour cream, 2 tablespoons of sugar, and ½ teaspoon of vanilla. Spread over the hot cheesecake, then bake until set, 3 to 5 minutes more. Let cool to room temperature in the pan, then refrigerate for at least 2 hours. Run a knife around the edge of the cheesecake before unmolding.\nM ake the cherry sauce: In a large skillet, combine the cherries, sugar, salt, and 2 tablespoons water. Cook over medium-high heat, stirring often, until the mixture begins to thicken, 4 to 6 minutes.\nIn a small bowl, stir together the cornstarch and 2 tablespoons water. Add to the cherries in the skillet and cook, stirring, until the mixture is thick and syrupy, 1 to 2 minutes. Stir in the lemon juice. Let cool completely. Serve with the cheesecake.",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
from recipe_scrapers._abstract import AbstractScraper
from recipe_scrapers._utils import get_minutes, normalize_string


class Food52(AbstractScraper):
    @classmethod
    def host(cls):
        return "food52.com"

    def title(self):
        return self.schema.title()

    def total_time(self):
        ul = self.soup.find("ul", {"class": "recipe__details"})
        total = 0
        for li in ul.find_all("li"):
            if li.span.get_text().lower() in ["prep time", "cook time"]:
                total += get_minutes(list(li.children)[2].strip())
        return total

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        instructions = self.soup.findAll("li", {"class": "recipe__list-step"})

        return "\n".join(
            [
                normalize_string(instruction.span.get_text())
                for instruction in instructions
            ]
        )

    def ratings(self):
        return self.schema.ratings()



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper


class RealFoodTesco(AbstractScraper):
    @classmethod
    def host(cls):
        return "realfood.tesco.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()



from recipe_scrapers._factory import SchemaScraperFactory
from tests import ScraperTest


class TestWildMode(ScraperTest):

    scraper_class = SchemaScraperFactory

    @classmethod
    def setUpClass(cls):
        with open("tests/test_data/wild_mode.testhtml", encoding="utf-8") as testfile:
            cls.harvester_class = cls.scraper_class.generate(
                url="https://test.example.com/", html=testfile.read()
            )

    def test_host(self):
        # let this one pass
        pass

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.feastingathome.com/tomato-risotto/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Summer Tomato Risotto with Saffron"
        )

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.feastingathome.com/wp-content/uploads/2020/07/Tomato-Risotto_-20-225x225.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 lb cherry or grape tomatoes",
                "1 tablespoon olive oil",
                "1 white or yellow onion, diced",
                "2 tablespoons olive oil",
                "4-6 cloves garlic, rough chopped",
                "1 teaspoon dried thyme (or 1 tablespoon fresh)",
                "1 1/2 cups arborio rice or Spanish short-grain rice",
                "pinch saffron",
                "1/2 teaspoon salt",
                "1/2 teaspoon pepper",
                "1/4 teaspoon smoked paprika",
                "6-8 cups veggie stock or chicken stock, warmed",
                "1 tablespoon butter",
                "1/4 cup grated parmesan",
                "16 ounces large shrimp -raw, peeled, deveined (or sub a white fish)",
                "1 tablespoon cumin",
                "1 tablespoon smoked paprika",
                "2 teaspoons granulated garlic",
                "1 teaspoon salt",
                "oil for searing",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            """BLISTER TOMATOES\nIn a large skillet, heat oil over medium-high heat. Add tomatoes (whole) and sear, stirring occasionally, until they burst and soften, about 7 minutes. Turn heat off. Chop if extra-large.\nMAKE RISOTTO\nAt the same time, in a large heavy-bottomed pot or dutch oven, heat the olive oil over medium heat and add the onions. Saute until golden about 10-12 minutes. Add garlic and thyme, saute 2 more minutes until fragrant.\nAdd the rice, saute 1 minute, stirring. Add 2 cups warm stock (enough to cover the rice), saffron and smoked paprika, stir and bring to a simmer. Simmer until most of the liquid is absorbed. Add 1 cup broth and the tomatoes and all their juices. Stir until all the liquid is absorbed. Continue adding broth 1 cup at a time, letting the rice absorb it slowly, stirring often over med-low heat, until the rice is plumped, slightly al dente, yet creamy, about 20-25 minutes. You may not need all 8 cups. ( I used 6 3/4).\nStir in the butter and parmesan. Season generously with salt, pepper, and optional chili flakes. Taste, adjust salt. If bland, it probably needs more salt.\nServe\nas a flavorful side or vegetarian main, garnishing with fresh parsley and lemon zest.\nOptional Seared Prawns:\nIf adding the prawns, mix spices and salt in a bowl. Coat shrimp with the spices. Heat 2-3 tablespoons oil in a skillet (you may need to do this in batches) over medium-high heat, sear each side 2-3 minutes or until cooked through. Top the risotto with the seared prawns.""",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        return self.assertEqual(4.9, self.harvester_class.ratings())

    def test_cuisine(self):
        return self.assertEqual("Spanish", self.harvester_class.cuisine())

    def test_description(self):
        return self.assertEqual(
            "Summer Tomato Risotto with juicy vine-ripened tomatoes, and infused with saffron, can be served as a vegetarian main or side dish, or topped off with smoky shrimp.",
            self.harvester_class.description(),
        )

    def test_total_time(self):
        return self.assertEqual(60, self.harvester_class.total_time())

    def test_cook_time(self):
        return self.assertEqual(45, self.harvester_class.cook_time())

    def test_prep_time(self):
        return self.assertEqual(15, self.harvester_class.prep_time())

    def test_author(self):
        return self.assertEqual("Sylvia Fountaine", self.harvester_class.author())



from recipe_scrapers.myrecipes import MyRecipes
from tests import ScraperTest


class TestMyRecipesScraper(ScraperTest):

    scraper_class = MyRecipes

    def test_host(self):
        self.assertEqual("myrecipes.com", self.harvester_class.host())

    def test_image(self):
        self.assertEqual(
            "https://imagesvc.meredithcorp.io/v3/mm/image?url=https%3A%2F%2Fcdn-image.myrecipes.com%2Fsites%2Fdefault%2Ffiles%2Fstyles%2F4_3_horizontal_-_1200x900%2Fpublic%2Fmyrecipestrending_1-09-18_1772_1.jpg%3Fitok%3DL_jRbjls%261516215251",
            self.harvester_class.image(),
        )

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.myrecipes.com/recipe/cacio-e-pepe",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Cacio e Pepe")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Adam Dolge")

    def test_total_time(self):
        self.assertEqual(20, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 items", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "6 oz. bucatini pasta",
                "1 1/2 tsp. kosher salt, divided",
                "3 Tbsp. olive oil",
                "1 tsp. black pepper, plus more for garnish",
                "1 1/2 oz. pecorino Romano, grated (about 3/4 cups), plus more for garnish",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Add pasta to a large skillet over high; cover with water, and add 1 teaspoon of the salt. Bring to a boil, and cook, stirring occasionally, until nearly tender, about 6 minutes. Meanwhile, heat oil in a medium skillet over medium; stir in black pepper, and cook until toasted, about 1 minute. Remove from heat. Whisk 3 tablespoons of the pasta cooking water into oil and pepper. Using tongs, transfer pasta into oil and pepper mixture, reserving pasta cooking water in skillet. Cook over low, stirring constantly, while sprinkling in cheese. Add more cooking water as needed, 1 tablespoon at a time, to create a creamy sauce. Stir in remaining 1/2 teaspoon salt. Divide pasta between 2 bowls, and garnish with pepper and cheese. Serve immediately.",
            self.harvester_class.instructions(),
        )

    def test_ratings_raises_exception(self):
        self.assertRaises(Exception, self.harvester_class.ratings)


# https://www.myrecipes.com/recipe/cacio-e-pepe
